
from tsgettoolbox.odo import odo, resource
import pandas as pd
import mando

try:
    from mando.rst_text_formatter import RSTHelpFormatter as HelpFormatter
except ImportError:
    from argparse import RawTextHelpFormatter as HelpFormatter

from tstoolbox import tsutils


@mando.command('ldas', formatter_class=HelpFormatter, doctype='numpy')
def ldas_cli(lat=None,
             lon=None,
             xindex=None,
             yindex=None,
             variable=None,
             startDate=None,
             endDate=None):
    r"""Download data from NLDAS or GLDAS.

    The time zone is always UTC.

    Parameters
    ----------
    lat :  float
        Either 'lat' and 'lon', or 'xindex' and
        'yindex' is required.

        Latitude (required): Enter single geographic point by
        latitude.::

            Example: --lat=43.1

    lon : float
        Either 'lat' and 'lon', or 'xindex' and
        'yindex' is required.  Longitude (required): Enter single
        geographic point by longitude::

            Example: --lon=-85.3

    xindex : int
        Either 'lat' and 'lon', or 'xindex' and
        'yindex' is required.  xindex (required if using xindex/yindex):
        Enter the x index of the NLDAS or GLDAS grid.::

            Example: --xindex=301

    yindex : int
        Either 'lat' and 'lon', or 'xindex' and
        'yindex' is required.  yindex (required if using xindex/yindex):
        Enter the y index of the NLDAS or GLDAS grid.::

            Example: --yindex=80

    variable : str
        Use the variable codes from the following table:

        +--------------------------------------------+-----------+
        | LDAS "variable" string Description         | Units     |
        +============================================+===========+
        | NLDAS:NLDAS_FORA0125_H.002:APCPsfc         | kg/m^2    |
        | Precipitation hourly total                 |           |
        +--------------------------------------------+-----------+
        | NLDAS:NLDAS_FORA0125_H.002:DLWRFsfc        | W/m^2     |
        | Surface DW longwave radiation flux         |           |
        +--------------------------------------------+-----------+
        | NLDAS:NLDAS_FORA0125_H.002:DSWRFsfc        | W/m^2     |
        | Surface DW shortwave radiation flux        |           |
        +--------------------------------------------+-----------+
        | NLDAS:NLDAS_FORA0125_H.002:PEVAPsfc        | kg/m^2    |
        | Potential evaporation                      |           |
        +--------------------------------------------+-----------+
        | NLDAS:NLDAS_FORA0125_H.002:SPFH2m          | kg/kg     |
        | 2-m above ground specific humidity         |           |
        +--------------------------------------------+-----------+
        | NLDAS:NLDAS_FORA0125_H.002:TMP2m           | degK      |
        | 2-m above ground temperature               |           |
        +--------------------------------------------+-----------+
        | NLDAS:NLDAS_FORA0125_H.002:UGRD10m         | m/s       |
        | 10-m above ground zonal wind               |           |
        +--------------------------------------------+-----------+
        | NLDAS:NLDAS_FORA0125_H.002:VGRD10m         | m/s       |
        | 10-m above ground meridional wind          |           |
        +--------------------------------------------+-----------+
        | NLDAS:NLDAS_NOAH0125_H.002:EVPsfc          | kg/m^2    |
        | Total evapotranspiration                   |           |
        +--------------------------------------------+-----------+
        | NLDAS:NLDAS_NOAH0125_H.002:GFLUXsfc        | w/m^2     |
        | Ground heat flux                           |           |
        +--------------------------------------------+-----------+
        | NLDAS:NLDAS_NOAH0125_H.002:LHTFLsfc        | w/m^2     |
        | Latent heat flux                           |           |
        +--------------------------------------------+-----------+
        | NLDAS:NLDAS_NOAH0125_H.002:SHTFLsfc        | w/m^2     |
        | Sensible heat flux                         |           |
        +--------------------------------------------+-----------+
        | NLDAS:NLDAS_NOAH0125_H.002:SSRUNsfc        | kg/m^2    |
        | Surface runoff (non-infiltrating)          |           |
        +--------------------------------------------+-----------+
        | NLDAS:NLDAS_NOAH0125_H.002:BGRIUNdfc       | kg/m^2    |
        | Subsurface runoff (baseflow)               |           |
        +--------------------------------------------+-----------+
        | NLDAS:NLDAS_NOAH0125_H.002:SOILM0-10cm     | kg/m^2    |
        | 0-10 cm soil moisture content              |           |
        +--------------------------------------------+-----------+
        | NLDAS:NLDAS_NOAH0125_H.002:SOILM0-100cm    | kg/m^2    |
        | 0-100 cm soil moisture content             |           |
        +--------------------------------------------+-----------+
        | NLDAS:NLDAS_NOAH0125_H.002:SOILM0-200cm    | kg/m^2    |
        | 0-200 cm soil moisture content             |           |
        +--------------------------------------------+-----------+
        | NLDAS:NLDAS_NOAH0125_H.002:SOILM10-40cm    | kg/m^2    |
        | 10-40 cm soil moisture content             |           |
        +--------------------------------------------+-----------+
        | NLDAS:NLDAS_NOAH0125_H.002:SOILM40-100cm   | kg/m^2    |
        | 40-100 cm soil moisture content            |           |
        +--------------------------------------------+-----------+
        | NLDAS:NLDAS_NOAH0125_H.002:SOILM100-200cm  | kg/m^2    |
        | 100-200 cm soil moisture content           |           |
        +--------------------------------------------+-----------+
        | NLDAS:NLDAS_NOAH0125_H.002:TSOIL0-10cm     | degK      |
        | 0-10 cm soil temperature                   |           |
        +--------------------------------------------+-----------+
        | GLDAS:GLDAS_NOAH025_3H.001:Evap            | kg/m^2/s  |
        | Evapotranspiration                         |           |
        +--------------------------------------------+-----------+
        | GLDAS:GLDAS_NOAH025_3H.001:precip          | kg/m^s/hr |
        | Precipitation rate                         |           |
        +--------------------------------------------+-----------+
        | GLDAS:GLDAS_NOAH025_3H.001:Rainf           | kg/m^2/s  |
        | Rain rate                                  |           |
        +--------------------------------------------+-----------+
        | GLDAS:GLDAS_NOAH025_3H.001:Snowf           | kg/m^2/s  |
        | Snow rate                                  |           |
        +--------------------------------------------+-----------+
        | GLDAS:GLDAS_NOAH025_3H.001:Qs              | kg/m^2/s  |
        | Surface Runoff                             |           |
        +--------------------------------------------+-----------+
        | GLDAS:GLDAS_NOAH025_3H.001:Qsb             | kg/m^2/s  |
        | Subsurface Runoff                          |           |
        +--------------------------------------------+-----------+
        | GLDAS:GLDAS_NOAH025_3H.001:SOILM0-100cm    | kg/m^2    |
        | 0-100 cm top 1 meter soil moisture content |           |
        +--------------------------------------------+-----------+
        | GLDAS:GLDAS_NOAH025_3H.001:SOILM0-10cm     | kg/m^2    |
        | 0-10 cm layer 1 soil moisture content      |           |
        +--------------------------------------------+-----------+
        | GLDAS:GLDAS_NOAH025_3H.001:SOILM10-40cm    | kg/m^2    |
        | 10-40 cm layer 2 soil moisture content     |           |
        +--------------------------------------------+-----------+
        | GLDAS:GLDAS_NOAH025_3H.001:SOILM40-100cm   | kg/m^2    |
        | 40-100 cm layer 3 soil moisture content    |           |
        +--------------------------------------------+-----------+
        | GLDAS:GLDAS_NOAH025_3H.001:Tair            | degK      |
        | Near surface air temperature               |           |
        +--------------------------------------------+-----------+
        | GLDAS:GLDAS_NOAH025_3H.001:TSOIL0-10cm     | degK      |
        | Average layer 1 soil temperature           |           |
        +--------------------------------------------+-----------+
        | GLDAS:GLDAS_NOAH025_3H.001:Wind            | m/s       |
        | Near surface wind magnitude                |           |
        +--------------------------------------------+-----------+

    startDate : str
        The start date of the time series.::

            Example: --startDate=2001-01-01T05

        If startDate and endDate are None, returns the entire series.

    endDate : str
        The end date of the time series.::

            Example: --startDate=2001-01-05T05

        If startDate and endDate are None, returns the entire series.
    """
    tsutils._printiso(ldas(lat=lat,
                           lon=lon,
                           xindex=xindex,
                           yindex=yindex,
                           variable=variable,
                           startDate=startDate,
                           endDate=endDate))


def ldas(lat=None,
         lon=None,
         xindex=None,
         yindex=None,
         variable=None,
         startDate=None,
         endDate=None):
    r"""Download data from NLDAS or GLDAS."""
    from tsgettoolbox.services import ldas as placeholder
    project = variable.split(':')[0]
    if lat is not None:
        location = 'GEOM:POINT({0}, {1})'.format(lon, lat)
    else:
        if project == 'NLDAS':
            location = '{0}:X{1:03d}-Y{2:03d}'.format(project, xindex, yindex)
        else:
            location = '{0}:X{1:04d}-Y{2:03d}'.format(project, xindex, yindex)

    r = resource(
        r'https://hydro1.gesdisc.eosdis.nasa.gov/daac-bin/access/timeseries.cgi',
        variable=variable,
        location=location,
        startDate=startDate,
        endDate=endDate,
    )
    return odo(r, pd.DataFrame)



from __future__ import absolute_import, division, print_function

from glob import glob
from .chunks import Chunks, chunks
from .resource import resource
from .utils import copydoc
from toolz import memoize, first
from datashape import discover, var
import os


class _Directory(Chunks):
    """ A directory of files on disk

    For typed containers see the ``Directory`` function which generates
    parametrized Directory classes.

    >>> from tsgettoolbox.odo import CSV
    >>> c = Directory(CSV)('path/to/data/')  # doctest: +SKIP

    Normal use through resource strings

    >>> r = resource('path/to/data/*.csv')  # doctest: +SKIP
    >>> r = resource('path/to/data/')  # doctest: +SKIP


    """
    def __init__(self, path, **kwargs):
        self.path = path
        self.kwargs = kwargs

    def __iter__(self):
        return (resource(os.path.join(self.path, fn), **self.kwargs)
                    for fn in sorted(os.listdir(self.path)))


@memoize
@copydoc(_Directory)
def Directory(cls):
    """ Parametrized DirectoryClass """
    return type('Directory(%s)' % cls.__name__, (_Directory, chunks(cls)), {})


re_path_sep = os.path.sep
if re_path_sep == '\\':
    re_path_sep = '\\\\'

@discover.register(_Directory)
def discover_Directory(c, **kwargs):
    return var * discover(first(c)).subshape[0]


@resource.register('.+' + re_path_sep + '\*\..+', priority=15)
def resource_directory(uri, **kwargs):
    path = uri.rsplit(os.path.sep, 1)[0]
    try:
        one_uri = first(glob(uri))
    except (OSError, StopIteration):
        return _Directory(path, **kwargs)
    subtype = type(resource(one_uri, **kwargs))
    return Directory(subtype)(path, **kwargs)


@resource.register('.+' + re_path_sep, priority=9)
def resource_directory_with_trailing_slash(uri, **kwargs):
    try:
        one_uri = os.listdir(uri)[0]
    except (OSError, IndexError):
        return _Directory(uri, **kwargs)
    subtype = type(resource(one_uri, **kwargs))
    return Directory(subtype)(uri, **kwargs)




import mando

try:
    from mando.rst_text_formatter import RSTHelpFormatter as HelpFormatter
except ImportError:
    from argparse import RawTextHelpFormatter as HelpFormatter

from tstoolbox import tsutils


@mando.command('usgs_eddn', formatter_class=HelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def usgs_eddn_cli(dcp_address,
                  parser,
                  start_date=None,
                  end_date=None):
    r"""Download from the USGS Emergency Data Distribution Network.

    This module provides access to data provided by the United States
    Geological Survey Emergency Data Distribution Network web site.

    The DCP message format includes some header information that is parsed and
    the message body, with a variable number of characters. The format of the
    message body varies widely depending on the manufacturer of the
    transmitter, data logger, sensors, and the technician who programmed the
    DCP. The body can be simple ASCII, sometime with parameter codes and
    time-stamps embedded, sometimes not. The body can also be in
    'Pseudo-Binary' which is character encoding of binary data that uses 6 bits
    of every byte and guarantees that all characters are printable.

    United States Geological Survey: http://www.usgs.gov/
    Emergency Data Distribution Network: http://eddn.usgs.gov/
    http://eddn.usgs.gov/dcpformat.html

    Fetches GOES Satellite DCP messages from USGS Emergency Data Distribution
    Network.

    Parameters
    ----------
    dcp_address
        DCP address or list of DCP addresses to be fetched; lists will be
        joined by a ','.

    parser
        Function that acts on dcp_message, where each row of the dataframe is
        processed and returns a new dataframe containing several rows of
        decoded data. This returned dataframe may have different (but derived)
        timestamps than that the original row. If a string is passed then
        a matching parser function is looked up from ulmo.usgs.eddn.parser.
        The prebuilt functions are "twdb_dot", "twdb_stevens", "twdb_sutron",
        and "twdb_texuni".

    {start_date}

    {end_date}
    """
    tsutils._printiso(usgs_eddn(dcp_address,
                                parser,
                                start_date=start_date,
                                end_date=end_date))


def usgs_eddn(dcp_address,
              parser,
              start_date=None,
              end_date=None):
    r"""Download from the USGS Emergency Data Distribution Network."""
    from tsgettoolbox.services.usgs import eddn

    df = eddn.ulmo_df(
        dcp_address=dcp_address,
        parser=parser,
        start_date=start_date,
        end_date=end_date)

    return df




from tsgettoolbox.odo import odo, resource
import pandas as pd
import mando

try:
    from mando.rst_text_formatter import RSTHelpFormatter as HelpFormatter
except ImportError:
    from argparse import RawTextHelpFormatter as HelpFormatter

from tstoolbox import tsutils


@mando.command('darksky', formatter_class=HelpFormatter, doctype='numpy')
def darksky_cli(latitude,
                longitude,
                time=None,
                database='hourly',
                extend=None,
                lang='en'):
    r"""Data from the Dark Sky forecast service.

    Powered by Dark Sky https://darksky.net/poweredby/

    Documentation: https://darksky.net/dev/docs

    Registration: https://darksky.net/dev/register

    The time zone of the returned data is dependent on the format of the "time"
    option.  If there is an ISO8601 representation of the time zone in the
    "time" option then that is the time zone of the returned data.  If "time"
    is None or does not explicitly define a time zone, the returned data is in
    the local time zone of the latitude and longitude.

    There isn't an absolutely consistent set of data returned for all areas, or
    all databases.  The returned values will be some subset of the following
    list:

    summary::

     A human-readable text summary of this data point.

    icon::

     A machine-readable text summary of this data |point, suitable for
     selecting an icon for display. If defined, this property will have one of
     the following values: 'clear-day', 'clear-night', 'rain', 'snow', 'sleet',
     'wind', 'fog', 'cloudy', 'partly-cloudy-day', or 'partly-cloudy-night'.
     (Developers should ensure that a sensible default is defined, as
     additional values, such as hail, thunderstorm, or tornado, may be defined
     in the future.)

    precipIntensity::

     A numerical value representing the average expected intensity (in
     inches of liquid water per hour) of precipitation occurring at the
     given time conditional on probability (that is, assuming any
     precipitation occurs at all). A very rough guide is that a value
     of 0 in./hr. corresponds to no precipitation, 0.002 in./hr.
     corresponds to very light precipitation, 0.017 in./hr. corresponds
     to light precipitation, 0.1 in./hr. corresponds to moderate
     precipitation, and 0.4 in./hr. corresponds to heavy precipitation.

    precipProbability::

     A numerical value between 0 and 1 (inclusive) representing the
     probability of precipitation occuring at the given time.

    precipType::

     A string representing the type of precipitation occurring at the
     given time. If defined, this property will have one of the
     following values: rain, snow, sleet (which applies to each of
     freezing rain, ice pellets, and 'wintery mix'), or hail. (If
     precipIntensity is zero, then this property will not be defined.)

    dewPoint::

     A numerical value representing the dew point at the given time in
     degrees Fahrenheit.

    windSpeed::

     A numerical value representing the wind speed in miles per hour.

    windBearing::

     A numerical value representing the direction that the wind is
     coming from in degrees, with true north at 0 degree and
     progressing clockwise. (If windSpeed is zero, then this value will
     not be defined.)

    cloudCover::

     A numerical value between 0 and 1 (inclusive) representing the
     percentage of sky occluded by clouds. A value of 0 corresponds to
     clear sky, 0.4 to scattered clouds, 0.75 to broken cloud cover,
     and 1 to completely overcast skies.

    humidity::

     A numerical value between 0 and 1 (inclusive) representing the
     relative humidity.

    pressure::

     A numerical value representing the sea-level air pressure in
     millibars.

    visibility::

     A numerical value representing the average visibility in miles,
     capped at 10 miles.

    ozone::

     A numerical value representing the columnar density of total
     atmospheric ozone at the given time in Dobson units.

    **Only defined for 'currently' data**

    nearestStormDistance::

     A numerical value representing the distance to the nearest storm
     in miles. (This value is very approximate and should not be used
     in scenarios requiring accurate results. In particular, a storm
     distance of zero doesn't necessarily refer to a storm at the
     requested location, but rather a storm in the vicinity of that
     location.)

    nearestStormBearing::

     A numerical value representing the direction of the nearest storm
     in degrees, with true north at 0 degree and progressing clockwise.
     (If nearestStormDistance is zero, then this value will not be
     defined. The caveats that apply to nearestStormDistance also apply
     to this value.)

    **Only defined for 'daily' data**

    sunriseTime/sunsetTime::

     The last sunrise before and first sunset after the solar noon
     closest to local noon on the given day.  (Note: near the poles,
     these may occur on a different day entirely!)

    moonPhase::

     A number representing the fractional part of the lunation number
     of the given day. This can be thought of as the 'percentage
     complete' of the current lunar month: a value of 0 represents
     a new moon, a value of 0.25 represents a first quarter moon,
     a value of 0.5 represents a full moon, and a value of 0.75
     represents a last quarter moon. (The ranges in between these
     represent waxing crescent, waxing gibbous, waning gibbous, and
     waning crescent moons, respectively.)

    precipIntensityMax, and precipIntensityMaxTime::

     Numerical values representing the maximumum expected intensity of
     precipitation on the given day in inches of liquid water per hour.

    temperatureMin, temperatureMinTime, temperatureMax,
    and temperatureMaxTime::

     Numerical values representing the minimum and maximumum
     temperatures (and the UNIX times at which they occur) on the given
     day in degrees Fahrenheit.

    apparentTemperatureMin, apparentTemperatureMinTime,
    apparentTemperatureMax, and
    apparentTemperatureMaxTime::

     Numerical values representing the minimum and maximumum apparent
     temperatures and the times at which they occur on the given day in
     degrees Fahrenheit.

    **Only defined for 'hourly' and 'daily' data**

    precipAccumulation::

     The amount of snowfall accumulation expected to occur on the given
     day, in inches. (If no accumulation is expected, this property
     will not be defined.)

    **Defined for every dataset except 'daily'**

    apparentTemperature::

     A numerical value representing the apparent (or 'feels like')
     temperature at the given time in degrees Fahrenheit.

    temperature::

     A numerical value representing the temperature at the given time
     in degrees Fahrenheit.

    Parameters
    ----------
    latitude : float
        Latitude (required): Enter single geographic point by
        latitude.

    longitude : float
        Longitude (required): Enter single geographic point by
        longitude.

    time
        TIME should either be a UNIX time (that is,
        seconds since midnight GMT on 1 Jan 1970) or a string formatted
        as follows: [YYYY]-[MM]-[DD]T[HH]:[MM]:[SS] (with an optional
        time zone formatted as Z for GMT time or {+,-}[HH][MM] for an
        offset in hours or minutes). For the latter format, if no
        timezone is present, local time (at the provided latitude and
        longitude) is assumed.  (This string format is a subset of ISO
        8601 time. An as example, 2013-05-06T12:00:00-0400.)

        The default is None, which uses the current time.

    database
        The database to draw the data from.  This is
        slightly different than the typical Forecast.io request, which
        would normally send back all data from all databases.  Typically
        though the 'tsgettoolbox' and siblings expect a single time
        increment in a dataset.  This isn't a hard rule, just
        a tradition.  So pick a database from 'currently', 'minutely',
        'hourly', 'daily', 'alerts', or 'flags'.  The 'currently'
        database is the default and is the current conditions.
        'minutely' give minute by minute forecast from the current time
        for the next hour, 'hourly' gives hourly forecast for the next
        two days (unless --extend='hourly' option is given), and 'daily'
        gives a forecast day by day for the next week.

    extend
        If set to 'hourly' and --database='hourly'
        then will get an hourly forecast for the next week.

    lang : str
        Return text summaries in the desired language.
        (Please be advised that units in the summary will be set
        according to the units option, above, so be sure to set both
        options as needed.)

        +-------------+-------------------+
        | lang= code  | Language          |
        +=============+===================+
        | ar          | Arabic            |
        +-------------+-------------------+
        | bs          | Bosnian           |
        +-------------+-------------------+
        | de          | German            |
        +-------------+-------------------+
        | en          | English (default) |
        +-------------+-------------------+
        | es          | Spanish           |
        +-------------+-------------------+
        | fr          | French            |
        +-------------+-------------------+
        | it          | Italian           |
        +-------------+-------------------+
        | nl          | Dutch             |
        +-------------+-------------------+
        | pl          | Polish            |
        +-------------+-------------------+
        | pt          | Portuguese        |
        +-------------+-------------------+
        | ru          | Russian           |
        +-------------+-------------------+
        | sk          | Slovak            |
        +-------------+-------------------+
        | sv          | Swedish           |
        +-------------+-------------------+
        | tet         | Tetum             |
        +-------------+-------------------+
        | tr          | Turkish           |
        +-------------+-------------------+
        | uk          | Ukrainian         |
        +-------------+-------------------+
        | x-pig-latin | Igpay Atinlay     |
        +-------------+-------------------+
        | zh          | Chinese           |
        +-------------+-------------------+

    """
    tsutils._printiso(darksky(latitude,
                              longitude,
                              time=time,
                              database=database,
                              extend=extend,
                              lang=lang))


def darksky(latitude,
            longitude,
            time=None,
            database='hourly',
            extend=None,
            lang='en'):
    r"""Data from the Dark Sky forecast service."""
    from tsgettoolbox.services import darksky as placeholder
    r = resource(
        r'https://api.darksky.net/forecast',
        latitude=latitude,
        longitude=longitude,
        database=database,
        time=time,
        extend=extend,
        units='si',
        lang=lang,
    )
    return odo(r, pd.DataFrame)


@mando.command(formatter_class=HelpFormatter, doctype='numpy')
def forecast_io(latitude,
                longitude,
                time=None,
                database='hourly',
                extend=None,
                lang='en'):
    r"""DEPRECATED: please use 'darksky'.

    The forecast_io service changed names to 'darksky'.  See documentation
    under the darksky service.

    Parameters
    ----------
    latitude
        See documentation under the darksky
        service.

    longitude
        See documentation under the darksky
        service.

    time
        See documentation under the darksky
        service.

    database
        See documentation under the darksky
        service.

    extend
        See documentation under the darksky
        service.

    lang
        See documentation under the darksky
        service.

    """
    return darksky(latitude,
                   longitude,
                   time=time,
                   database=database,
                   extend=extend,
                   lang=lang)



from __future__ import print_function
from __future__ import absolute_import

import logging
import os
from builtins import object

from tsgettoolbox.odo import convert
from tsgettoolbox.odo import odo
from tsgettoolbox.odo import resource

import pandas as pd

import requests

from tsgettoolbox import utils

from tstoolbox import tsutils

# darksky.net


class darksky_net_json(object):
    def __init__(self, url, **query_params):
        self.url = url
        self.url_params = {}
        self.url_params['latitude'] = query_params.pop('latitude')
        self.url_params['longitude'] = query_params.pop('longitude')
        self.url_params['time'] = tsutils.parsedate(
            query_params.pop('time'),
            strftime='%Y-%m-%dT%H:%M:%S')
        self.include_db = query_params.pop('database')
        all_dbs = ['currently', 'minutely',
                   'hourly', 'daily', 'alerts', 'flags']
        all_dbs.remove(self.include_db)
        query_params['exclude'] = ','.join(all_dbs)
        query_params['units'] = 'si'
        self.query_params = query_params


@resource.register(r'https://api\.darksky\.net/forecast.*', priority=17)
def resource_darksky_net(uri, **kwargs):
    return darksky_net_json(uri, **kwargs)

# Function to convert from darksky_io_json type to pd.DataFrame


@convert.register(pd.DataFrame, darksky_net_json)
def darksky_net_json_to_df(data, **kwargs):
    # Read in API key
    api_key = utils.read_api_key('darksky.net')

    urlvar = '{0},{1}'.format(data.url_params['latitude'],
                              data.url_params['longitude'])

    req = requests.get('/'.join([data.url,
                                 api_key,
                                 urlvar]),
                       data.query_params)

    if os.path.exists('debug_tsgettoolbox'):
        logging.warning(req.url)
    req.raise_for_status()

    time_zone_name = req.json()['timezone']
    try:
        ndfj = pd.read_json(req.content, orient='index')
    except ValueError:
        return pd.DataFrame()

    if isinstance(ndfj.ix[0, 0], dict):
        ndfj.ix[0, 0] = [ndfj.ix[0, 0]]

    ndfj = pd.DataFrame(ndfj.ix[data.include_db, :])
    ndfj = ndfj.transpose()

    ndfj.dropna(inplace=True, how='all')

    if data.include_db in ['minutely', 'hourly', 'daily']:
        ndfj = pd.DataFrame(ndfj.ix[data.include_db, 'data'])
    elif data.include_db in ['alerts']:
        ndfj = pd.DataFrame(ndfj.ix[data.include_db, 0])

    if data.include_db != 'flags':
        ndfj.index = pd.to_datetime(ndfj['time'], unit='s')
        ndfj.drop('time', axis=1, inplace=True)
        ndfj.sort_index(inplace=True)
        ndfj = ndfj.tz_localize(time_zone_name)

    for datecols in ['apparentTemperatureMinTime',
                     'apparentTemperatureMaxTime',
                     'apparentTemperatureLowTime',
                     'apparentTemperatureHighTime',
                     'precipIntensityMaxTime',
                     'sunriseTime',
                     'sunsetTime',
                     'temperatureMaxTime',
                     'temperatureMinTime',
                     'uvIndexTime',
                     'windGustTime',
                     ]:
        if datecols in ndfj.columns:
            ndfj[datecols] = pd.to_datetime(ndfj[datecols], unit='s')

    ndfj.index.name = 'Datetime:{0}'.format(time_zone_name)
    unitsd = {'nearestStormDistance': ':km',
              'precipIntensity': ':mm/h',
              'precipIntensityMax': ':mm/h',
              'precipAccumulation': ':cm',
              'temperature': ':degC',
              'temperatureMin': ':degC',
              'temperatureMax': ':degC',
              'apparentTemperature': ':degC',
              'apparentTemperatureLow': ':degC',
              'apparentTemperatureHigh': ':degC',
              'apparentTemperatureMin': ':degC',
              'apparentTemperatureMax': ':degC',
              'dewPoint': ':degC',
              'windSpeed': ':m/s',
              'windGust': ':m/s',
              'pressure': ':hPa',
              'visibility': ':km',
              'ozone': 'DU'}
    ndfj.columns = ['{0}{1}'.format(i, unitsd.setdefault(i, ''))
                    for i in ndfj.columns]
    return ndfj


if __name__ == '__main__':
    r = resource(
        r'https://api.darksky.net/forecast',
        latitude=28.45,
        longitude=-81.34,
        database='currently',
        time='2020-01-01T01:00:00',
    )

    as_df = odo(r, pd.DataFrame)
    print('darksky.net currently')
    print(as_df.head())

    r = resource(
        r'https://api.darksky.net/forecast',
        latitude=28.45,
        longitude=-81.34,
        database='minutely',
        time=None,
    )

    as_df = odo(r, pd.DataFrame)
    print('darksky.net minutely')
    print(as_df.head())

    r = resource(
        r'https://api.darksky.net/forecast',
        latitude=28.45,
        longitude=-81.34,
        database='hourly',
        time=None,
    )

    as_df = odo(r, pd.DataFrame)
    print('darksky.net hourly')
    print(as_df.head())

    r = resource(
        r'https://api.darksky.net/forecast',
        latitude=28.45,
        longitude=-81.34,
        database='daily',
        time=None,
    )

    as_df = odo(r, pd.DataFrame)
    print('darksky.net daily')
    print(as_df.head())

    r = resource(
        r'https://api.darksky.net/forecast',
        latitude=28.45,
        longitude=-81.34,
        database='alerts',
        time=None,
    )

    as_df = odo(r, pd.DataFrame)
    print('darksky.net alerts')
    print(as_df.head())

    r = resource(
        r'https://api.darksky.net/forecast',
        latitude=28.45,
        longitude=-81.34,
        database='flags',
        time=None,
    )

    as_df = odo(r, pd.DataFrame)
    print('darksky.net flags')
    print(as_df.head())

    r = resource(
        r'https://api.darksky.net/forecast',
        latitude=28.45,
        longitude=-81.34,
        database='currently',
        time='yesterday',
    )

    as_df = odo(r, pd.DataFrame)
    print('darksky.net flags')
    print(as_df.head())



#!/usr/bin/env python
r"""
tsgettoolbox command line/library tools to retrieve time series.

This program is a collection of utilities to download data from various
web services.
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os.path
import sys
import warnings

import mando

try:
    from mando.rst_text_formatter import RSTHelpFormatter as HelpFormatter
except ImportError:
    from argparse import RawTextHelpFormatter as HelpFormatter

from tstoolbox import tsutils

warnings.filterwarnings('ignore')


@mando.command()
def about():
    r"""Print out information about tsgettoolbox and the system."""
    tsutils.about(__name__)


from .functions.cdec import cdec
from .functions.coops import coops
from .functions.cpc import cpc
from .functions.darksky import darksky
from .functions.darksky import forecast_io
from .functions.daymet import daymet
from .functions.lcra import lcra_hydromet
from .functions.lcra import lcra_wq
from .functions.ldas import ldas
from .functions.modis import modis
from .functions.ncdc import ncdc_annual
from .functions.ncdc import ncdc_ghcnd
from .functions.ncdc import ncdc_ghcnd_ftp
from .functions.ncdc import ncdc_ghcndms
from .functions.ncdc import ncdc_gs
from .functions.ncdc import ncdc_nexrad2
from .functions.ncdc import ncdc_nexrad3
from .functions.ncdc import ncdc_normal_ann
from .functions.ncdc import ncdc_normal_dly
from .functions.ncdc import ncdc_normal_hly
from .functions.ncdc import ncdc_normal_mly
from .functions.ncdc import ncdc_precip_15
from .functions.ncdc import ncdc_precip_hly
from .functions.ndbc import ndbc
from .functions.nwis import epa_wqp
from .functions.nwis import nwis
from .functions.nwis import nwis_dv
from .functions.nwis import nwis_gwlevels
from .functions.nwis import nwis_iv
from .functions.nwis import nwis_measurements
from .functions.nwis import nwis_peak
from .functions.nwis import nwis_site
from .functions.nwis import nwis_stat
from .functions.twc import twc
from .functions.unavco import unavco
from .functions.usgs_eddn import usgs_eddn


def main():
    r"""Main function."""
    if not os.path.exists('debug_tsgettoolbox'):
        sys.tracebacklimit = 0
    mando.main()


if __name__ == '__main__':
    main()




from __future__ import absolute_import

import logging
import os
import time

from tsgettoolbox.odo import convert
from tsgettoolbox.odo import odo
from tsgettoolbox.odo import resource

import pandas as pd

from requests import Request, Session
from requests.utils import unquote

from tsgettoolbox import utils

from tstoolbox import tstoolbox
from tstoolbox import tsutils

# ncdc_cdo


class ncdc_cdo_json(object):
    def __init__(self, url, **query_params):
        self.url = url
        self.query_params = query_params
        self.query_params['limit'] = 1000
        self.query_params['units'] = 'metric'


@resource.register(
    r'http://www\.ncdc\.noaa\.gov/cdo-web/api/v2/datasets*',
    priority=17)
@resource.register(
    r'http://www\.ncdc\.noaa\.gov/cdo-web/api/v2/datacategories*',
    priority=17)
@resource.register(
    r'http://www\.ncdc\.noaa\.gov/cdo-web/api/v2/datatypes*',
    priority=17)
@resource.register(
    r'http://www\.ncdc\.noaa\.gov/cdo-web/api/v2/locationcategories*',
    priority=17)
@resource.register(
    r'http://www\.ncdc\.noaa\.gov/cdo-web/api/v2/locations*',
    priority=17)
@resource.register(
    r'http://www\.ncdc\.noaa\.gov/cdo-web/api/v2/stations*',
    priority=17)
@resource.register(
    r'http://www\.ncdc\.noaa\.gov/cdo-web/api/v2/data/*',
    priority=17)
def resource_ncdc_cdo(uri, **kwargs):
    return ncdc_cdo_json(uri, **kwargs)

# Function to convert from ncdc_cdo_json type to pd.DataFrame


@convert.register(pd.DataFrame, ncdc_cdo_json)
def ncdc_cdo_json_to_df(data, **kwargs):
    # Read in API key
    api_key = utils.read_api_key('ncdc_cdo')

    headers = {'token': api_key}

    sdate = pd.datetime(1900, 1, 1)
    td = pd.datetime.today()
    edate = pd.datetime(td.year, td.month, td.day) + pd.Timedelta(days=1)
    if 'NORMAL_' in data.query_params['datasetid']:
        # All the NORMAL_* datasets must have a startdate/endate of
        # 2010-01-01/2010-12-31
        sdate = pd.datetime(2010, 1, 1)
        edate = pd.datetime(2010, 12, 31)
        delta = edate - sdate
    elif 'stationid' in data.query_params:
        # Get startdate and/or enddate information
        s = Session()
        ireq = Request('GET',
                       r'http://www.ncdc.noaa.gov/cdo-web/api/v2/stations/{0}'.format(
                           data.query_params['stationid']),
                       headers=headers)
        prepped = ireq.prepare()
        dreq = s.send(prepped)

        sdate = pd.to_datetime(dreq.json()['mindate'])
        edate = pd.to_datetime(dreq.json()['maxdate'])

        if 'startdate' in data.query_params:
            tdate = tsutils.parsedate(data.query_params['startdate'])
            if tdate > sdate:
                sdate = tdate

        if 'enddate' in data.query_params:
            tdate = tsutils.parsedate(data.query_params['enddate'])
            if tdate < edate:
                edate = tdate
        delta = pd.Timedelta(days=365)
    else:
        delta = pd.Timedelta(days=106751)

    if sdate >= edate:
        raise ValueError("""
*
*   The startdate of {0} is greater than, or equal to, the enddate of {1}.
*
""".format(sdate, edate))

    df = pd.DataFrame()

    testdate = sdate
    while testdate < edate:
        time.sleep(1)

        data.query_params['startdate'] = testdate.strftime('%Y-%m-%d')

        testdate = testdate + delta
        if testdate > edate:
            testdate = edate

        data.query_params['enddate'] = testdate.strftime('%Y-%m-%d')

        s = Session()
        ireq = Request('GET',
                       data.url,
                       params=data.query_params,
                       headers=headers)
        prepped = ireq.prepare()
        prepped.url = unquote(prepped.url)
        if os.path.exists('debug_tsgettoolbox'):
            logging.warning(prepped.url)
        req = s.send(prepped)
        req.raise_for_status()

        try:
            tdf = pd.io.json.json_normalize(req.json()['results'])
        except KeyError:
            continue

        tdf.set_index('date', inplace=True)
        tdf.index = pd.to_datetime(tdf.index)
        df = df.combine_first(tdf)

    if len(df) == 0:
        if 'NORMAL_' in data.query_params['datasetid']:
            raise ValueError("""
*
*   No normalized statistics available for station {0}
*
""".format(data.query_params['stationid']))
        else:
            raise ValueError("""
*
*   No data within {0} and {1}.
*   There should be data between {2} and {3}.
*
""".format(data.query_params['startdate'],
                data.query_params['enddate'],
                pd.to_datetime(dreq.json()['mindate']),
                pd.to_datetime(dreq.json()['maxdate'])))

    df = df.drop('station', axis='columns')
    df = tstoolbox.unstack('datatype',
                           input_ts=df)
    return df


if __name__ == '__main__':
    # http://www.ncdc.noaa.gov/cdo-web/api/v2/data?
    #  datasetid=PRECIP_15&
    #  stationid=COOP:010008&
    #  units=metric&startdate=2010-05-01&enddate=2010-05-31
    r = resource(
        r'http://www.ncdc.noaa.gov/cdo-web/api/v2/data',
        startdate='2010-05-01',
        enddate='2010-05-31',
        stationid='COOP:010008',
        datasetid='PRECIP_15',
    )
    as_df = odo(r, pd.DataFrame)
    print(as_df)
    mardi = [
        ['GHCND', 'GHCND:AE000041196'],
        ['GHCND', 'GHCND:USR0000GCOO'],
        ['PRECIP_HLY', 'COOP:087440'],
        ['PRECIP_15', 'COOP:087440'],
        # ['ANNUAL', 'GHCND:US1MOLN0006'],
        ['GHCNDMS', 'GHCND:US1FLAL0004'],
        ['GSOM', 'GHCND:US1FLAL0004'],
        ['GSOY', 'GHCND:USW00012816'],
        # ['NORMAL_ANN', 'GHCND:USC00083322'],
        ['NORMAL_HLY', 'GHCND:USW00013889'],
        ['NORMAL_DLY', 'GHCND:USC00084731'],
        ['NORMAL_MLY', 'GHCND:USC00086618'],
        # ['NEXRAD3', 'NEXRAD:KJAX'],
        # ['NEXRAD2', 'NEXRAD:KJAX'],
    ]
    for did, sid, in mardi:
        startdate = '2010-01-01'
        enddate = '2013-01-01'
        if 'NEXRAD' in did:
            startdate = '2000-01-01'
        if 'PRECIP_' in did:
            startdate = '2009-01-01'

        r = resource(
            r'http://www.ncdc.noaa.gov/cdo-web/api/v2/data',
            startdate=startdate,
            stationid=sid,
            datasetid=did,
        )

        as_df = odo(r, pd.DataFrame)
        print(did)
        print(as_df)



"""
Provide access to data from the USACE `Rivergages`_ web site.

`United States Army Corps of Engineers`_ `Rivergages`_ web site.

.. _United States Army Corps of Engineers: http://www.usace.army.mil/
.. _Rivergages: http://rivergages.mvr.usace.army.mil/WaterControl/new/layout.cfm
"""
import pandas as pd

from tsgettoolbox.ulmo.usace.rivergages.core import get_station_data
from tsgettoolbox.ulmo.usace.rivergages.core import get_station_parameters
from tsgettoolbox.ulmo.usace.rivergages.core import get_stations

# def get_station_data(station_code, parameter, start=None, end=None,
#         min_value=None, max_value=None):


def ulmo_df(station_code,
            parameter,
            start_date=None,
            end_date=None):
    tstations = get_stations()
    if station_code not in tstations:
        raise ValueError("""
*
*   Station code {0} not in available stations:
*   {1}
*
""".format(station_code, tstations.keys))

    tparameters = get_station_parameters(station_code)
    if parameter not in tparameters:
        raise ValueError("""
*
*   Parameter code {0} not in available parameters at station {1}:
*   {2}
*
""".format(parameter, station_code, tparameters))
    df = get_station_data(station_code,
                          parameter,
                          start=pd.to_datetime(start_date),
                          end=pd.to_datetime(end_date))
    df = pd.DataFrame.from_dict(df, orient='index')
    df.sort_index(inplace=True)
    df.index.name = 'Datetime'
    df.columns = ['{0}_{1}'.format(station_code, parameter)]
    return df


if __name__ == '__main__':
    #    import time
    #
    #    r = ulmo_df('blah',
    #                'upperbasin')
    #
    #    print('BIVOI_HL')
    #    print(r)
    #
    r = ulmo_df('BIVO1',
                'HL',
                start_date='2015-11-04',
                end_date='2015-12-05')

    print('BIVOI HL')
    print(r)



from __future__ import print_function
from __future__ import absolute_import

import logging
import os
from builtins import object
from io import BytesIO

from tsgettoolbox.odo import convert
from tsgettoolbox.odo import odo
from tsgettoolbox.odo import resource

import pandas as pd

import requests

from tstoolbox import tsutils

_UNITS_MAP = {
    'NLDAS:NLDAS_FORA0125_H.002:APCPsfc':
        ['Precipitation hourly total', 'kg/m^2'],
    'NLDAS:NLDAS_FORA0125_H.002:DLWRFsfc':
        ['Surface DW longwave radiation flux', 'W/m^2'],
    'NLDAS:NLDAS_FORA0125_H.002:DSWRFsfc':
        ['Surface DW shortwave radiation flux', 'W/m^2'],
    'NLDAS:NLDAS_FORA0125_H.002:PEVAPsfc':
        ['Potential evaporation', 'kg/m^2'],
    'NLDAS:NLDAS_FORA0125_H.002:SPFH2m':
        ['2-m above ground specific humidity', 'kg/kg'],
    'NLDAS:NLDAS_FORA0125_H.002:TMP2m':
        ['2-m above ground temperature', 'degK'],
    'NLDAS:NLDAS_FORA0125_H.002:UGRD10m':
        ['10-m above ground zonal wind', 'm/s'],
    'NLDAS:NLDAS_FORA0125_H.002:VGRD10m':
        ['10-m above ground meridional wind', 'm/s'],
    'NLDAS:NLDAS_NOAH0125_H.002:EVPsfc':
        ['Total evapotranspiration', 'kg/m^2'],
    'NLDAS:NLDAS_NOAH0125_H.002:GFLUXsfc':
        ['Ground heat flux', 'w/m^2'],
    'NLDAS:NLDAS_NOAH0125_H.002:LHTFLsfc':
        ['Latent heat flux', 'w/m^2'],
    'NLDAS:NLDAS_NOAH0125_H.002:SHTFLsfc':
        ['Sensible heat flux', 'w/m^2'],
    'NLDAS:NLDAS_NOAH0125_H.002:SSRUNsfc':
        ['Surface runoff (non-infiltrating)', 'kg/m^2'],
    'NLDAS:NLDAS_NOAH0125_H.002:BGRUNsfc':
        ['Subsurface runoff (baseflow)', 'kg/m^2'],
    'NLDAS:NLDAS_NOAH0125_H.002:SOILM0-10cm':
        ['0-10 cm soil moisture content', 'kg/m^2'],
    'NLDAS:NLDAS_NOAH0125_H.002:SOILM0-100cm':
        ['0-100 cm soil moisture content', 'kg/m^2'],
    'NLDAS:NLDAS_NOAH0125_H.002:SOILM0-200cm':
        ['0-200 cm soil moisture content', 'kg/m^2'],
    'NLDAS:NLDAS_NOAH0125_H.002:SOILM10-40cm':
        ['10-40 cm soil moisture content', 'kg/m^2'],
    'NLDAS:NLDAS_NOAH0125_H.002:SOILM40-100cm':
        ['40-100 cm soil moisture content', 'kg/m^2'],
    'NLDAS:NLDAS_NOAH0125_H.002:SOILM100-200cm':
        ['100-200 cm soil moisture content', 'kg/m^2'],
    'NLDAS:NLDAS_NOAH0125_H.002:TSOIL0-10cm':
        ['0-10 cm soil temperature', 'degK'],

    'GLDAS2:GLDAS_NOAH025_3H_v2.0:ESoil_tavg':
        ['Direct evaporation from bare soil', 'w/m2'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.0:Evap_tavg':
        ['Evapotranspiration', 'kg/m2/s'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.0:LWdown_f_tavg':
        ['Downward long-wave radiation flux', 'W/m2'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.0:PotEvap_tavg':
        ['Potential evaporation rate', 'W/m2'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.0:Psurf_f_inst':
        ['Pressure', 'Pa'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.0:Qair_f_inst':
        ['Specific humidity', 'kg/kg'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.0:Qs_acc':
        ['Storm surface runoff', 'kg/m2'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.0:Qsb_acc':
        ['Baseflow-groundwater runoff', 'kg/m2'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.0:Qsm_acc':
        ['Snow melt', 'kg/m2'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.0:Rainf_f_tavg':
        ['Total precipitation rate', 'kg/m2/s'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.0:Rainf_tavg':
        ['Rain precipitation rate', 'kg/m2/s'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.0:RootMoist_inst':
        ['Root zone soil moisture', 'kg/m2'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.0:Snowf_tavg':
        ['Snow precipitation rate', 'kg/m2/s'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.0:SoilMoi0_10cm_inst':
        ['Soil moisture content (0-10 cm underground)', 'kg/m2'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.0:SoilMoi10_40cm_inst':
        ['Soil moisture content (10-40 cm underground)', 'kg/m2'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.0:SoilMoi40_100cm_inst':
        ['Soil moisture content (40-100 cm underground)', 'kg/m2'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.0:SoilMoi100_200cm_inst':
        ['Soil moisture content (100-200 cm underground)', 'kg/m2'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.0:SoilTMP0_10cm_inst':
        ['Soil temperature (0-10 cm underground)', 'K'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.0:SoilTMP10_40cm_inst':
        ['Soil temperature (10-40 cm underground)', 'K'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.0:SWdown_f_tavg':
        ['Downward short-wave radiation flux', 'W/m2'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.0:Tair_f_inst':
        ['Temperature', 'K'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.0:Wind_f_inst':
        ['Wind speed', 'm/s'],

    'GLDAS2:GLDAS_NOAH025_3H_v2.1:AvgSurfT_inst':
        ['Average surface skin temperature', 'K'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.1:Evap_tavg':
        ['Evapotranspiration', 'kg/m2/s'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.1:Psurf_f_inst':
        ['Surface air pressure', 'Pa'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.1:Qair_f_inst':
        ['Specific humidity', 'kg/kg'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.1:Qs_acc':
        ['Storm surface runoff', 'kg/m2'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.1:Qsb_acc':
        ['Baseflow-groundwater runoff', 'kg/m2'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.1:Rainf_f_tavg':
        ['Total precipitation rate', 'kg/m2/s'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.1:Rainf_tavg':
        ['Rain precipitation rate', 'kg/m2/s'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.1:Snowf_tavg':
        ['Snow precipitation rate', 'kg/m2/s'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.1:SoilMoi0_10cm_inst':
        ['Soil moisture content (0-10 cm underground)', 'kg/m2'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.1:SoilMoi10_40cm_inst':
        ['Soil moisture content (10-40 cm underground)', 'kg/m2'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.1:SoilMoi40_100cm_inst':
        ['Soil moisture content (40-100 cm underground)', 'kg/m2'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.1:SoilTMP0_10cm_inst':
        ['Soil temperature (0-10 cm underground)', 'K'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.1:Tair_f_inst':
        ['Near surface air temperature', 'K'],
    'GLDAS2:GLDAS_NOAH025_3H_v2.1:Wind_f_inst':
        ['Near surface wind speed', 'm/s'],
}

# LDAS


class LDAS(object):
    def __init__(self, url, **query_params):
        query_params['type'] = 'asc2'
        self.url = url
        self.query_params = query_params
        self.query_params['startDate'] = tsutils.parsedate(
            self.query_params['startDate'], strftime='%Y-%m-%dT%H')
        self.query_params['endDate'] = tsutils.parsedate(
            self.query_params['endDate'], strftime='%Y-%m-%dT%H')

# Function to make `resource` know about the new Daymet type.
# http://hydro1.gesdisc.eosdis.nasa.gov/daac-bin/access/timeseries.cgi?variable=GLDAS:GLDAS_NOAH025_3H.001:SOILM10-40cm&location=GEOM:POINT%28-99.875,%2031.125%29&startDate=2010-06-01T09&endDate=2015-05-04T21&type=asc2
#
# https://hydro1.gesdisc.eosdis.nasa.gov/daac-bin/access/timeseries.cgi?variable=NLDAS:NLDAS_FORA0125_H.002:APCPsfc&location=NLDAS:X304-Y071&startDate=2015-01-01T00&endDate=2015-06-20T23&type=asc2
#  http://hydro1.gesdisc.eosdis.nasa.gov/daac-bin/access/timeseries.cgi?variable=NLDAS:NLDAS_FORA0125_H.002:APCPsfc&location=NLDAS:X304-Y071&startDate=2015-01-01T00&endDate=2015-06-20T23&type=asc2


@resource.register(r'https://hydro1\.gesdisc\.eosdis\.nasa\.gov/daac-bin/access/timeseries\.cgi.*', priority=17)
def resource_ldas(uri, **kwargs):
    return LDAS(uri, **kwargs)


def _parse_ldas_dates(date, hour):
    try:
        return pd.to_datetime(date) + pd.to_timedelta(pd.np.int(hour[:-1]), 'h')
    except (TypeError, ValueError):
        return pd.NaT

# Function to convert from LDAS type to pd.DataFrame


@convert.register(pd.DataFrame, LDAS)
def ldas_to_df(data, **kwargs):
    req = requests.get(data.url,
                       params=data.query_params)
    if os.path.exists('debug_tsgettoolbox'):
        logging.warning(req.url)
    req.raise_for_status()

    df = pd.read_table(BytesIO(req.content),
                       skiprows=40,
                       header=None,
                       index_col=None,
                       delim_whitespace=True)
    df.drop(df.index[-1], axis='rows', inplace=True)
    if len(df.columns) == 3:
        df['dt'] = df[0] + "T" + df[1]
        df['dt'] = pd.to_datetime(df['dt'])
        df.set_index('dt', inplace=True)
        df.drop([0, 1], axis='columns', inplace=True)
    else:
        df[0] = pd.to_datetime(df[0])
        df.set_index(0, inplace=True)
    variable_name = data.query_params['variable'].split(':')[-1]
    unit = _UNITS_MAP[data.query_params['variable']][1]
    df.columns = ['{0}:{1}'.format(variable_name, unit)]
    df.index.name = 'Datetime:UTC'
    return df.tz_localize('UTC')


if __name__ == '__main__':
    # ?variable=GLDAS:GLDAS_NOAH025_3H.001:SOILM10-40cm&
    # location=GEOM:POINT%28-99.875,%2031.125%29&
    # startDate=2010-06-01T09&endDate=2015-05-04T21&type=asc2
    #
    for key in _UNITS_MAP:
        r = resource(
            r'https://hydro1.gesdisc.eosdis.nasa.gov/daac-bin/access/timeseries.cgi',
            variable=key,
            location='GEOM:POINT(-99.875, 31.125)',
            startDate='2010-06-01T09',
            endDate='2011-05-04T21'
        )

        as_df = odo(r, pd.DataFrame)
        print('LDAS', key)
        print(as_df)
#
#     r = resource(
#         r'https://hydro1.gesdisc.eosdis.nasa.gov/daac-bin/access/timeseries.cgi',
#         variable='GLDAS:GLDAS_NOAH025_3H.001:SOILM10-40cm',
#         location='GEOM:POINT(104.2, 35.86)',
#         startDate='2016-01-01T00',
#         endDate='2016-12-01T00'
#     )
#
#     as_df = odo(r, pd.DataFrame)
#     print('LDAS TEST')
#     print(as_df)
#
#     r = resource(
#         r'https://hydro1.gesdisc.eosdis.nasa.gov/daac-bin/access/timeseries.cgi',
#         variable='GLDAS2:GLDAS_NOAH025_3H_v2.1:SOILM10-40cm',
#         location='GEOM:POINT(104.2, 35.86)',
#         startDate='5 years ago',
#         endDate='4 years ago'
#     )
#
#     as_df = odo(r, pd.DataFrame)
#     print('LDAS TEST')
#     print(as_df)



"""
Access hydrologic and climate data from the `Lower Colorado River Authority`_.

Data for the Colorado River Basin (Texas) provided by the `Lower Colorado River
Authority`_ `Hydromet`_ web site and web service.

.. _Lower Colorado River Authority: http://www.lcra.org
.. _Hydromet: http://hydromet.lcra.org
"""
from __future__ import print_function

from builtins import str

import pandas as pd

from tsgettoolbox.ulmo.lcra.hydromet.core import get_current_data
from tsgettoolbox.ulmo.lcra.hydromet.core import get_site_data


def ulmo_df(site_code,
            parameter_code,
            start_date=None,
            end_date=None,
            dam_site_location='head'):
    if parameter_code.lower() in ['upperbasin', 'lowerbasin']:
        # def get_current_data(service, as_geojson=False):
        df = pd.DataFrame(get_current_data('get' + parameter_code))
        return df.set_index('location')

    # def get_site_data(site_code, parameter_code, as_dataframe=True,
    #     start_date=None, end_date=None, dam_site_location='head'):
    return get_site_data(str(site_code),
                         parameter_code,
                         start_date=pd.to_datetime(start_date),
                         end_date=pd.to_datetime(end_date),
                         dam_site_location=dam_site_location)


if __name__ == '__main__':
    #    import time
    #
    #    r = ulmo_df('blah',
    #                'upperbasin')
    #
    #    print('UB EVERYTHING')
    #    print(r)
    #
    #    time.sleep(20)

    r = ulmo_df(4598,
                'stage',
                start_date='2015-11-04',
                end_date='2015-12-05')

    print('UB EVERYTHING')
    print(r)



"""
Access to data provided by the `Lower Colorado River Authority`_.

`Water Quality`_ web site.
.. _Lower Colorado River Authority: http://www.lcra.org
.. _Water Quality: http://waterquality.lcra.org/
"""
from __future__ import print_function

from tsgettoolbox.ulmo.lcra.waterquality.core import get_historical_data, get_recent_data

# def get_historical_data(site_code, start=None, end=None, as_dataframe=False):
# def get_recent_data(site_code, as_dataframe=False):


def ulmo_df(site_code,
            start_date=None,
            end_date=None,
            historical=True):
    """Call to ulmo.lcra.waterquality.core functions.

    Specifically: .get_historical_data or .get_recent_data.
    """
    if historical is True:
        return get_historical_data(site_code,
                                   start=start_date,
                                   end=end_date,
                                   as_dataframe=True)
    return get_recent_data(site_code,
                           as_dataframe=True)


if __name__ == '__main__':
    #    import time
    #
    #    r = ulmo_df('blah',
    #                'upperbasin')
    #
    #    print('UB EVERYTHING')
    #    print(r)
    #
    #    time.sleep(20)

    R = ulmo_df(6977,
                historical=True,
                start_date='2015-11-04',
                end_date='2015-12-05')

    print('UB EVERYTHING')
    print(R)



from __future__ import print_function
from __future__ import absolute_import

import datetime
from base64 import b64decode
from gzip import GzipFile
import logging
import os
from builtins import object
from io import StringIO
from io import BytesIO

from tsgettoolbox.odo import convert
from tsgettoolbox.odo import odo
from tsgettoolbox.odo import resource

import pandas as pd

import requests

from tstoolbox import tsutils


class NDBC(object):
    def __init__(self, url, **query_params):
        self.lmap = {'stdmet': 'h',
                     'cwind': 'c',
                     'wlevel': 'l',
                     'ocean': 'o',
                     'adcp': 'a',
                     'supl': 's',
                     'srad': 'r'}

        # 'wlevel' header is manipulated and not in self.headermap
        self.headermap = {'stdmet': {'WDIR': 'WDIR:degT',
                                     'WSPD': 'WSPD:m/s',
                                     'GST': 'GST:m/s',
                                     'WVHT': 'WVHT:m',
                                     'DPD': 'DPD:s',
                                     'APD': 'APD:s',
                                     'MWD': 'MWD:degT',
                                     'PRES': 'PRES:hPa',
                                     'ATMP': 'ATMP:degC',
                                     'WTMP': 'WTMP:degC',
                                     'DEWP': 'DEWP:degC',
                                     'VIS': 'VIS:nautical_miles',
                                     'PTDY': 'PTDY:hPa',
                                     'TIDE': 'TIDE:ft',
                                     },
                          'cwind': {'WDIR': 'WDIR:degT',
                                    'WSPD': 'WSPD:m/s',
                                    'GDR': 'GDR:degT',
                                    'GST': 'GST:m/s',
                                    'GTIME': 'GTIME'
                                    },
                          'ocean': {'DEPTH': 'DEPTH:m',
                                    'OTMP': 'OTMP:degC',
                                    'COND': 'COND:mS/cm',
                                    'SAL': 'SAL:PSU',
                                    'O2%': 'O2:percent',
                                    'O2PPM': 'O2PPM:ppm',
                                    'CLCON': 'CLCON:ug/l',
                                    'TURB': 'TURB:FTU',
                                    'PH': 'PH:',
                                    'EH': 'EH:mv',
                                    },
                          'adcp': {'DEP01': 'DEP01:m',
                                   'DIR01': 'DIR01:degT',
                                   'SPD01': 'SPD01:cm/s',
                                   'DEP02': 'DEP02:m',
                                   'DIR02': 'DIR02:degT',
                                   'SPD02': 'SPD02:cm/s',
                                   'DEP03': 'DEP03:m',
                                   'DIR03': 'DIR03:degT',
                                   'SPD03': 'SPD03:cm/s',
                                   'DEP04': 'DEP04:m',
                                   'DIR04': 'DIR04:degT',
                                   'SPD04': 'SPD04:cm/s',
                                   'DEP05': 'DEP05:m',
                                   'DIR05': 'DIR05:degT',
                                   'SPD05': 'SPD05:cm/s',
                                   'DEP06': 'DEP06:m',
                                   'DIR06': 'DIR06:degT',
                                   'SPD06': 'SPD06:cm/s',
                                   'DEP07': 'DEP07:m',
                                   'DIR07': 'DIR07:degT',
                                   'SPD07': 'SPD07:cm/s',
                                   'DEP08': 'DEP08:m',
                                   'DIR08': 'DIR08:degT',
                                   'SPD08': 'SPD08:cm/s',
                                   'DEP09': 'DEP09:m',
                                   'DIR09': 'DIR09:degT',
                                   'SPD09': 'SPD09:cm/s',
                                   'DEP10': 'DEP10:m',
                                   'DIR10': 'DIR10:degT',
                                   'SPD10': 'SPD10:cm/s',
                                   'DEP11': 'DEP11:m',
                                   'DIR11': 'DIR11:degT',
                                   'SPD11': 'SPD11:cm/s',
                                   'DEP12': 'DEP12:m',
                                   'DIR12': 'DIR12:degT',
                                   'SPD12': 'SPD12:cm/s',
                                   'DEP13': 'DEP13:m',
                                   'DIR13': 'DIR13:degT',
                                   'SPD13': 'SPD13:cm/s',
                                   'DEP14': 'DEP14:m',
                                   'DIR14': 'DIR14:degT',
                                   'SPD14': 'SPD14:cm/s',
                                   'DEP15': 'DEP15:m',
                                   'DIR15': 'DIR15:degT',
                                   'SPD15': 'SPD15:cm/s',
                                   'DEP16': 'DEP16:m',
                                   'DIR16': 'DIR16:degT',
                                   'SPD16': 'SPD16:cm/s',
                                   'DEP17': 'DEP17:m',
                                   'DIR17': 'DIR17:degT',
                                   'SPD17': 'SPD17:cm/s',
                                   'DEP18': 'DEP18:m',
                                   'DIR18': 'DIR18:degT',
                                   'SPD18': 'SPD18:cm/s',
                                   'DEP19': 'DEP19:m',
                                   'DIR19': 'DIR19:degT',
                                   'SPD19': 'SPD19:cm/s',
                                   'DEP20': 'DEP20:m',
                                   'DIR20': 'DIR20:degT',
                                   'SPD20': 'SPD20:cm/s',
                                   },
                          'srad': {'SRAD1': 'SRAD1:w/m2',
                                   'SWRAD': 'SWRAD:w/m2',
                                   'LWRAD': 'LWRAD:w/m2',
                                   }
                          }
        self.rename = {
                'WD': 'WDIR:degT',
                'DIR': 'WDIR:degT',
                'SPD': 'WSPD:m/s',
                'GSP': 'GST:m/s',
                'GMN': 'GTIME',
                'BARO': 'PRES:hPa',
                'H0': 'WVHT:m',
                'DOMPD': 'DPD:s',
                'AVP': 'APD:s',
                'SRAD': 'SWRAD',
                'SRAD2': 'SWRAD',
                'LRAD': 'LWRAD',
                'LRAD1': 'LWRAD',
                'BAR': 'PRES:hPa',
                }
        for item in self.headermap:
            self.rename.update(self.headermap[item])

        self.url = url
        self.query_params = query_params


# Function to make `resource` know about the new NOS type.
#  https://www.ndbc.noaa.gov/download_data.php?
#      filename=41013h2003.txt.gz&
#      dir=data/historical/stdmet/
#  https://www.ndbc.noaa.gov/download_data.php?
#      filename=4101322018.txt.gz&
#      dir=data/stdmet/Feb/
#  https://www.ndbc.noaa.gov/data/historical/stdmet/41012h2012.txt.gz
#  https://www.ndbc.noaa.gov/data/stdmet/Mar/sauf132018.txt.gz
@resource.register(r'https://www\.ndbc\.noaa\.gov/data/*', priority=17)
def resource_ndbc(uri, **kwargs):
    return NDBC(uri, **kwargs)


def date_parser(*x):
    x = [int(i) for i in x]
    if x[0] < 100:
        x[0] = x[0] + 1900
    return pd.datetime(*x)


mapnumtoname = {0: 'Jan',
                1: 'Feb',
                2: 'Mar',
                3: 'Apr',
                4: 'May',
                5: 'Jun',
                6: 'Jul',
                7: 'Aug',
                8: 'Sep',
                9: 'Oct',
                10: 'Nov',
                11: 'Dec'}


# Function to convert from NDBC type to pd.DataFrame
@convert.register(pd.DataFrame, NDBC)
def ndbc_to_df(data, **kwargs):

    sdate = tsutils.parsedate(data.query_params.pop('startUTC'))
    edate = tsutils.parsedate(data.query_params.pop('endUTC'))

    table = data.query_params['table']

    df = pd.DataFrame()

    cyear = datetime.datetime.now()
    filenames = []
    for yr in range(sdate.year, edate.year + 1):
        # Yearly
        # https://www.ndbc.noaa.gov/data/historical/stdmet/41012h2012.txt.gz
        filenames.append('/historical/{3}/{0}{1}{2}.txt.gz'.format(
            data.query_params['station'],
            data.lmap[table],
            yr,
            table))
    if edate.year == cyear.year:
        for mnth in range(edate.month):
            # Monthly
            # https://www.ndbc.noaa.gov/data/stdmet/Mar/sauf132018.txt.gz
            filenames.append('/{3}/{4}/{0}{1}{2}.txt.gz'.format(
                data.query_params['station'],
                mnth + 1,
                yr,
                table,
                mapnumtoname[mnth]))

    for filename in filenames:
        req = requests.get(data.url + filename)

        if os.path.exists('debug_tsgettoolbox'):
            logging.warning(req.url)

        try:
            req.raise_for_status()
        except requests.exceptions.HTTPError:
            continue

        content = BytesIO(req.content)
        content = GzipFile(fileobj=content).read()

        # Test to see if 1 or 2 line header
        f = StringIO(content.decode('utf-8'))
        head1 = f.readline()
        head2 = f.readline()
        f.seek(0)

        words1 = head1.split()
        words2 = head2.split()
        try:
            _ = int(words2[0])
            skiprows = None
        except ValueError:
            skiprows = [1]

        if 'mm' == words1[4]:
            parse_dates = {'datetime': [0, 1, 2, 3, 4]}
        else:
            parse_dates = {'datetime': [0, 1, 2, 3]}

        tdf = pd.read_csv(f,
                          header=0,
                          skiprows=skiprows,
                          sep=r'\s+',
                          parse_dates=parse_dates,
                          date_parser=date_parser,
                          index_col=0,
                          na_values=['MM', 999.0, 99.0, 9999, 99999])

        if len(tdf) > 0:
            tdf.rename(columns=data.rename, inplace=True)
            df = df.append(tdf)

    if len(df) == 0:
        raise ValueError("""
*
*   No data collected/available within this time frame.
*
""")

    if table == 'wlevel':
        df.columns = list(range(0, 55, 6))
        df = pd.DataFrame(df.stack())
        df.index = [i + datetime.timedelta(minutes=j) for i, j in df.index]
        df.index.name = 'Datetime'
        df.columns = ['WLEVEL:ft:MLLW']

    # Clean up the dataframe...
    df = df.sort_index()
    df = df[~df.index.duplicated()]

    return df.loc[sdate:edate, :]


if __name__ == '__main__':
    """
    """

    r = resource(
        r'https://www.ndbc.noaa.gov/data/',
        table='stdmet',
        startUTC='2012-01-01T00:00Z',
        endUTC='2012-04-01:00Z',
        station='41012',
    )

    as_df = odo(r, pd.DataFrame)
    print('NDBC')
    print(as_df)
    as_df.to_csv('file.csv')




import mando

try:
    from mando.rst_text_formatter import RSTHelpFormatter as HelpFormatter
except ImportError:
    from argparse import RawTextHelpFormatter as HelpFormatter

from tstoolbox import tsutils


@mando.command('twc', formatter_class=HelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def twc_cli(county,
            start_date=None,
            end_date=None):
    r"""Download Texas Weather Connection (TWC) data.

    This module provides direct access to `Texas Weather Connection`_ `Daily
    Keetch-Byram Drought Index (KBDI)`_ dataset.

    .. _Texas Weather Connection: http://twc.tamu.edu/
    .. _Daily Keetch-Byram Drought Index (KBDI): http://twc.tamu.edu/drought/kbdi

    Parameters
    ----------
    county: ``None`` or str
        If specified, results will be limited to the county corresponding to
        the given 5-character Texas county fips code i.e. 48.
    {start_date}
    {end_date}
    """
    tsutils._printiso(twc(county,
                          start_date=start_date,
                          end_date=end_date))


def twc(county,
        start_date=None,
        end_date=None):
    r"""Download Texas Weather Connection (TWC) data."""
    from tsgettoolbox.services import twc

    df = twc.ulmo_df(county=county,
                     start_date=start_date,
                     end_date=end_date)

    return df



from __future__ import absolute_import
from __future__ import print_function

import re
from builtins import str
from builtins import zip

import pandas as pd

DEFAULT_START_DATE = '01/01/1901'
DEFAULT_END_DATE = 'Now'


def get_stations():
    """Fetch information on all CDEC sites.

    Returns
    -------
    df : pandas DataFrame
        a pandas DataFrame (indexed on site id) with station information.

    """
    # I haven't found a better list of stations, seems pretty janky
    # to just have them in a file, and not sure if/when it is updated.
    url = 'http://cdec.water.ca.gov/misc/all_stations.csv'
    # the csv is malformed, so some rows think there are 7 fields
    col_names = ['id', 'meta_url', 'name', 'num', 'lat', 'lon', 'junk']
    df = pd.read_csv(url,
                     names=col_names,
                     header=None,
                     quotechar="'",
                     index_col=0)

    return df


def get_sensors(sensor_id=None):
    """
    Get a list of sensor ids as a DataFrame indexed on sensor number.

    Can be limited by a list of numbers.

    Usage example::

        from ulmo import cdec
        # to get all available sensor info
        sensors = cdec.historical.get_sensors()
        # or to get just one sensor
        sensor = cdec.historical.get_sensors([1])

    Parameters
    ----------
    sites : iterable of integers or ``None``

    Returns
    -------
    df : pandas DataFrame
        a python dict with site codes mapped to site information

    """
    url = 'http://cdec.water.ca.gov/misc/senslist.html'
    df = pd.read_html(url, header=0)[0]
    df.set_index('Sensor No')

    if sensor_id is None:
        return df
    return df.ix[sensor_id]


def get_station_sensors(station_ids=None, sensor_ids=None, resolutions=None):
    """Get available sensors for the given stations.

    Get available sensors, sensor ids and time resolutions. If no station ids
    are provided, all available stations will be used (this is not recommended,
    and will probably take a really long time).

    The list can be limited by a list of sensor numbers, or time resolutions
    if you already know what you want. If none of the provided sensors or
    resolutions are available, an empty DataFrame will be returned for that
    station.

    Usage example::

        from ulmo import cdec
        # to get all available sensors
        available_sensors = cdec.historical.get_station_sensors(['NEW'])

    Parameters
    ----------
    station_ids : iterable of strings or ``None``

    sensor_ids : iterable of integers or ``None``
        check out  or use the ``get_sensors()`` function to see a list of
        available sensor numbers

    resolutions : iterable of strings or ``None``
        Possible values are 'event', 'hourly', 'daily', and 'monthly' but not
        all of these time resolutions are available at every station.

    Returns
    -------
    dict : a python dict
        a python dict with site codes as keys with values containing pandas
        DataFrames of available sensor numbers and metadata.

    """
    station_sensors = {}

    if station_ids is None:
        station_ids = get_stations().index

    for station_id in station_ids:
        url = 'http://cdec.water.ca.gov/cgi-progs/queryCSV?station_id={0}'.format(station_id)

        sensor_list = pd.read_html(url)[0]
        sensor_list.columns = ['sensor_id',
                               'variable',
                               'resolution',
                               'timerange']
        v = list(sensor_list.variable.to_dict().values())

        split = [re.split(r'[\(\)]+', x) for x in v]
        var_names = [x[0] for x in split]
        units = [x[1] for x in split]
        var_resolution = [re.split(r'[\(\)]+', x)[1]
                          for x in sensor_list.resolution]

        sensor_list['resolution'] = var_resolution
        sensor_list['variable'] = [x + y
                                   for x, y in zip(var_names, var_resolution)]
        sensor_list['units'] = pd.Series(units,
                                         index=sensor_list.index)

        station_sensors[station_id] = _limit_sensor_list(sensor_list,
                                                         sensor_ids,
                                                         resolutions)

    return station_sensors


def get_data(station_ids=None,
             sensor_ids=None,
             resolutions=None,
             start=None,
             end=None):
    """Download data for a set of CDEC station and sensor ids.

    If either is not provided, all available data will be downloaded. Be really
    careful with choosing hourly resolution as the data sets are big, and
    CDEC's servers are slow as molasses in winter.

    Usage example::

        from ulmo import cdec
        dat = cdec.historical.get_data(['PRA'],resolutions=['daily'])

    Parameters
    ----------
    station_ids : iterable of strings or ``None``

    sensor_ids : iterable of integers or ``None``
        check out  or use the ``get_sensors()`` function to see a list of
        available sensor numbers

    resolutions : iterable of strings or ``None``
        Possible values are 'event', 'hourly', 'daily', and 'monthly' but not
        all of these time resolutions are available at every station.

    Returns
    -------
    dict : a python dict
        a python dict with site codes as keys. Values will be nested dicts
        containing all of the sensor/resolution combinations.

    """
    if start is None:
        start_date = pd.Timestamp(DEFAULT_START_DATE).date()
    else:
        start_date = pd.Timestamp(start).date()
    if end is None:
        end_date = pd.Timestamp(DEFAULT_END_DATE).date()
    else:
        end_date = pd.Timestamp(end).date()

    start_date_str = '{0}/{1}/{2}'.format(start_date.month,
                                          start_date.day,
                                          start_date.year)
    end_date_str = '{0}/{1}/{2}'.format(end_date.month,
                                        end_date.day,
                                        end_date.year)

    if station_ids is None:
        station_ids = get_stations().index

    sensors = get_station_sensors(station_ids, sensor_ids, resolutions)

    d = {}

    for station_id, sensor_list in list(sensors.items()):
        station_data = {}

        for _, row in sensor_list.iterrows():
            res = row.ix['resolution']
            var = row.ix['variable']
            sensor_id = row.ix['sensor_id']

            url = 'http://cdec.water.ca.gov/cgi-progs/queryCSV' + \
                  '?station_id=' + station_id + \
                  '&dur_code=' + res_to_dur_code_map[res] + \
                  '&sensor_num=' + str(sensor_id) + \
                  '&start_date=' + start_date_str + \
                  '&end_date=' + end_date_str
            station_data[var] = pd.read_csv(url,
                                            skiprows=2,
                                            header=None,
                                            parse_dates=[[0, 1]],
                                            index_col=None,
                                            na_values='m')
            station_data[var].columns = ['datetime', 'value']
            station_data[var].set_index('datetime', inplace=True)

        d[station_id] = station_data

    return d


def _limit_sensor_list(sensor_list, sensor_ids, resolution):

    if sensor_ids is not None:
        sensor_list = sensor_list[[x in sensor_ids
                                   for x in sensor_list.sensor_id]]

    if resolution is not None:
        sensor_list = sensor_list[[x in resolution
                                   for x in sensor_list.resolution]]

    return sensor_list


res_to_dur_code_map = {'hourly': 'H',
                       'daily': 'D',
                       'monthly': 'M',
                       'event': 'E'}


def ulmo_df(station_id,
            sensor_num=None,
            dur_code=None,
            start_date=None,
            end_date=None):

    if isinstance(sensor_num, (str, bytes)):
        sensor_num = [int(i) for i in sensor_num.split('.')]
    elif isinstance(sensor_num, int):
        sensor_num = [sensor_num]

    if isinstance(dur_code, (str, bytes)):
        dur_code = dur_code.split(',')

        mapd = {'D': 'daily',
                'E': 'event',
                'H': 'hourly',
                'M': 'monthly'}

        dur_code = [mapd[i] for i in dur_code]

    d = get_data([station_id],
                 sensor_ids=sensor_num,
                 resolutions=dur_code,
                 start=start_date,
                 end=end_date)

    nd = pd.DataFrame()
    for key in d[station_id]:
        nnd = pd.DataFrame(d[station_id][key])
        nnd.columns = [key]
        nd = nd.join(nnd, how='outer')
    nd.rename(lambda x:
              x.replace(',', '_').replace(' ', '_').replace('__', '_'),
              axis='columns',
              inplace=True)
    nd = nd.tz_localize('UTC').tz_convert('Etc/GMT+8')
    nd.index.name = 'Datetime:PST'
    return nd


if __name__ == '__main__':
    r = ulmo_df('PAR',
                start_date='2017-01-01',
                end_date='2017-10-02',
                dur_code='D',
                sensor_num=45)

    print('PAR PRECIPITATION')
    print(r)

    r = ulmo_df('PAR',
                start_date='2017-01-01',
                end_date='2017-10-02',
                dur_code='H',
                sensor_num=6)

    print('PAR RESERVOIR VOLUME')
    print(r)

    r = ulmo_df('PAR',
                start_date='2017-01-01',
                end_date='2017-10-02',
                dur_code='H')

    print('PAR HOURLY')
    print(r)

    r = ulmo_df('PAR',
                start_date='2017-01-01',
                end_date='2017-10-02')

    print('PAR EVERYTHING')
    print(r)



#!/usr/bin/env python
r"""
tsgettoolbox command line/library tools to retrieve time series.

This program is a collection of utilities to download data from various
web services.
"""

from __future__ import print_function
from __future__ import division
from __future__ import absolute_import

import warnings

from tsgettoolbox.odo import odo, resource
import pandas as pd
import mando

try:
    from mando.rst_text_formatter import RSTHelpFormatter as HelpFormatter
except ImportError:
    from argparse import RawTextHelpFormatter as HelpFormatter

from tstoolbox import tsutils

warnings.filterwarnings('ignore')


@mando.command('daymet', formatter_class=HelpFormatter, doctype='numpy')
def daymet_cli(lat,
               lon,
               measuredParams=None,
               year=None):
    r"""Download data from Daymet by the Oak Ridge National Laboratory.

    Detailed documentation is at http://daymet.ornl.gov/.  Since this is
    daily data, it covers midnight to midnight based on local time.

    Parameters
    ----------
    lat : float
        Latitude (required): Enter single geographic point by latitude, value
        between 52.0N and 14.5N.::

            Example: --lat=43.1

    lon : float
        Longitude (required): Enter single geographic point by longitude, value
        between -131.0W and -53.0W.::

            Example: --lon=-85.3

    measuredParams:  CommaSeparatedVariables (optional)
        Use the abbreviations from the following table:

        +----------------+-----------------------+---------+
        | measuredParams | Description           | Unit    |
        +================+=======================+=========+
        | tmax           | maximum temperature   | degC    |
        +----------------+-----------------------+---------+
        | tmin           | minimum temperature   | degC    |
        +----------------+-----------------------+---------+
        | srad           | shortwave radiation   | W/m2    |
        +----------------+-----------------------+---------+
        | vp             | vapor pressure        | Pa      |
        +----------------+-----------------------+---------+
        | swe            | snow-water equivalent | kg/m2   |
        +----------------+-----------------------+---------+
        | prcp           | precipitation         | mm      |
        +----------------+-----------------------+---------+
        | dayl           | daylength             | seconds |
        +----------------+-----------------------+---------+

        Example: --measuredParams=tmax,tmin

        All variables are returned by default.
    year :  CommaSeparatedYears (optional):
        Current Daymet product (version 2) is available from 1980 to the latest
        full calendar year.::

            Example: --years=2012,2013

        All years are returned by default.
    """
    tsutils._printiso(daymet(lat,
                             lon,
                             measuredParams=measuredParams,
                             year=year))


def daymet(lat,
           lon,
           measuredParams=None,
           year=None):
    r"""Download data from Daymet by the Oak Ridge National Laboratory."""
    from tsgettoolbox.services import daymet as placeholder
    r = resource(
        r'http://daymet.ornl.gov/data/send/saveData',
        measuredParams=measuredParams,
        lat=lat,
        lon=lon,
        year=year,
    )
    return odo(r, pd.DataFrame)




import mando
from tsgettoolbox.odo import odo, resource
import pandas as pd

try:
    from mando.rst_text_formatter import RSTHelpFormatter as HelpFormatter
except ImportError:
    from argparse import RawTextHelpFormatter as HelpFormatter

from tstoolbox import tsutils


@mando.command('coops', formatter_class=HelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def coops_cli(station,
              date=None,
              begin_date=None,
              end_date=None,
              range=None,
              product='water_level',
              datum='NAVD',
              time_zone='GMT',
              interval='h',
              bin=None):
    r"""Download Center for Operational Oceanographic Products and Services.

    CO-OPS web services is at http://tidesandcurrents.noaa.gov/api/.  The time
    zone of the returned data depends on the setting of the "time_zone" option.
    The default is "GMT" also known as "UTC".

    Parameters
    ----------
    station
        A 7 character station ID, or a currents station ID.  Specify the
        station ID with the "station=" parameter.::

            Example: '--station=9414290'

        Station listings for various products can be viewed at
        http://tidesandcurrents.noaa.gov or viewed on a map at Tides & Currents
        Station Map

    date : str
        [optional, default is None]

        The API understands several parameters related to date ranges.  Date
        formats are pretty flexible, however the closer to ISO 8601, the
        better.

        The date related options are 'begin_date', 'end_date', 'date', and
        'range'.  They can be combined in the following 5 ways, but if
        conflicting then follows the table in order.  For example, the 'date'
        option will be used if present regardless of any other option, then
        'range', ...etc.:

        +--------------------+---------------------------------------+
        | Parameter Name(s)  | Description                           |
        +====================+=======================================+
        | '--date'           | 'latest', 'today', or 'recent'        |
        +--------------------+---------------------------------------+
        | '--range'          | Specify a number of hours to go back  |
        |                    | from now and retrieve data for that   |
        |                    | date range                            |
        +--------------------+---------------------------------------+
        | '--begin_date' and | Specify a begin date and a number of  |
        | '--range'          | hours to retrieve data starting from  |
        |                    | that date                             |
        +--------------------+---------------------------------------+
        | '--begin_date' and | Specify the date/time range of        |
        | '--end_date'       | retrieval                             |
        +--------------------+---------------------------------------+
        | '--end_date' and a | Specify an end date and a number of   |
        | '--range'          | hours to retrieve data ending at that |
        |                    | date                                  |
        +--------------------+---------------------------------------+

        +-------------------+--------------------------------------+
        | Maximum Retrieval | Data Types                           |
        +===================+======================================+
        | 31 days           | All 6 minute data products           |
        +-------------------+--------------------------------------+
        | 1 year            | Hourly Height, and High/Low          |
        +-------------------+--------------------------------------+
        | 10 years          | Tide Predictions, Daily, and Monthly |
        |                   | Means                                |
        +-------------------+--------------------------------------+

        +--------------------------------------+-----------------+
        | Description of "--date" option       | Option          |
        +======================================+=================+
        | Today's data                         | --date='today'  |
        +--------------------------------------+-----------------+
        | The last 3 days of data              | --date='recent' |
        +--------------------------------------+-----------------+
        | The last data point available within | --date='latest' |
        | the last 18 minutes.                 |                 |
        +--------------------------------------+-----------------+

    begin_date
        [optional, default is None]

        The beginning date for the data.  See explanation with the 'date'
        option on how to use all of the date related parameters.

    end_date
        [optional, default is None]

        The end date for the data.  January 1st, 2012 through January 2nd,
        2012::

            --begin_date='20120101' --end_date='20120102'

        See explanation with the 'date' option on how to use all of the date
        related parameters.

    range
        [optional, default is None]

        Specify the number of hours to go back from now, an 'end_date', or
        forward from a 'begin_date'.

        48 hours beginning on April 15, 2012::

            --begin_date='20120415' --range=48

        48 hours ending on March 17, 2012::

            --end_date='20120307' --range=48

        The last 24 hours from now::

            --range=24

        The last 3 hours from now::

            --range=3

        See explanation with the 'date' option on how to use all of the date
        related parameters.

    product : str
        [optional, default is 'water_level']

        Specify the observation requested.  If not specified will download all
        data from that station.

        +------------------------+-------------------------------------+
        | Option                 | Description                         |
        +========================+=====================================+
        | water_level            | Preliminary or verified water       |
        |                        | levels, depending on availability.  |
        +------------------------+-------------------------------------+
        | air_temperature        | Air temperature                     |
        +------------------------+-------------------------------------+
        | water_temperature      | Water temperature                   |
        +------------------------+-------------------------------------+
        | wind                   | Wind speed, direction, and gusts    |
        +------------------------+-------------------------------------+
        | air_gap                | (distance between a bridge and the  |
        |                        | water's surface)                    |
        +------------------------+-------------------------------------+
        | conductivity           | The water's conductivity            |
        +------------------------+-------------------------------------+
        | visibility             | Visibility from the station's       |
        |                        | visibility sensor. A measure of     |
        |                        | atmospheric clarity.                |
        +------------------------+-------------------------------------+
        | humidity               | Relative humidity                   |
        +------------------------+-------------------------------------+
        | salinity               | Salinity and specific gravity       |
        +------------------------+-------------------------------------+
        | hourly_height          | Verified hourly height water level  |
        |                        | data                                |
        +------------------------+-------------------------------------+
        | high_low               | Verified high/low water level data  |
        +------------------------+-------------------------------------+
        | daily_mean             | Verified daily mean water level     |
        |                        | data                                |
        +------------------------+-------------------------------------+
        | monthly_mean           | Verified monthly mean water level   |
        |                        | data                                |
        +------------------------+-------------------------------------+
        | one_minute_water_level | One minute water level data         |
        +------------------------+-------------------------------------+
        | predictions            | 6 minute predictions water level    |
        |                        | data                                |
        +------------------------+-------------------------------------+
        | datums                 | datums data                         |
        +------------------------+-------------------------------------+
        | currents               | Currents data                       |
        +------------------------+-------------------------------------+

        Possible water level height variables:

        +-------+----------------------------------------------------------+
        | Name  | Description                                              |
        +=======+==========================================================+
        | Sigma | Standard deviation of 1 second samples used to compute   |
        |       | the water level height                                   |
        +-------+----------------------------------------------------------+
        | O     | Count of number of samples that fall outside a 3-sigma   |
        |       | band about the mean                                      |
        +-------+----------------------------------------------------------+
        | I     | A flag that indicates that the water level value has     |
        |       | been inferred.                                           |
        +-------+----------------------------------------------------------+
        | F     | A flag that when set to 1 indicates that the flat        |
        |       | tolerance limit was exceeded                             |
        +-------+----------------------------------------------------------+
        | R     | A flag that when set to 1 indicates that the rate of     |
        |       | change tolerance limit was exceeded                      |
        +-------+----------------------------------------------------------+
        | T     | A flag that when set to 1 indicates that the temperature |
        |       | difference tolerance limit was exceeded                  |
        +-------+----------------------------------------------------------+
        | L     | A flag that when set to 1 indicates that either the      |
        |       | maximum or minimum expected water level height limit     |
        |       | was exceeded                                             |
        +-------+----------------------------------------------------------+

        Meteorological and ancillary data variables:

        +------+------------------------------------------------------+
        | Name | Description                                          |
        +======+======================================================+
        | X    | A flag that when set to 1 indicates that the maximum |
        |      | value was exceeded                                   |
        +------+------------------------------------------------------+
        | N    | A flag that when set to 1 indicates that the minimum |
        |      | expected value was exceeded                          |
        +------+------------------------------------------------------+
        | R    | A flag that when set to 1 indicates that the rate of |
        |      | change tolerance limit was exceeded                  |
        +------+------------------------------------------------------+

    datum
        [optional, default is 'NAVD']

        Specify the datum that all water levels will be
        reported against.  Note! Datum is mandatory for all water level
        products.

        +--------+-------------------------------+
        | Option | Description                   |
        +========+===============================+
        | MHHW   | Mean Higher High Water        |
        +--------+-------------------------------+
        | MHW    | Mean High Water               |
        +--------+-------------------------------+
        | MTL    | Mean Tide Level               |
        +--------+-------------------------------+
        | MSL    | Mean Sea Level                |
        +--------+-------------------------------+
        | MLW    | Mean Low Water                |
        +--------+-------------------------------+
        | MLLW   | Mean Lower Low Water          |
        +--------+-------------------------------+
        | NAVD   | North American Vertical Datum |
        +--------+-------------------------------+
        | STND   | Station Datum                 |
        +--------+-------------------------------+

    time_zone
        [optional, default is 'GMT']

        The time zone is specified as 'gmt', 'lst'
        or 'lst_ldt'.

        +---------+----------------------------------------------------+
        | Option  | Description                                        |
        +=========+====================================================+
        | gmt     | Greenwich Mean Time                                |
        +---------+----------------------------------------------------+
        | lst     | Local Standard Time. The time local to the         |
        |         | requested station.                                 |
        +---------+----------------------------------------------------+
        | lst_ldt | Local Standard/Local Daylight Time. The time local |
        |         | to the requested station.                          |
        +---------+----------------------------------------------------+

    interval
        [optional, defaults to 'h']

        Deliver the Meteorological data at hourly intervals.  Does not override
        6 minute intervals for --product='water_level'.

    bin
        [optional, defaults to None]

        The bin number for the specified currents station Example:'--bin=4'
        Will retrieve data for bin number 4. Note! If a bin is not specified
        for a PORTS station, the data is returned using a predefined real-time
        bin.

    """
    tsutils._printiso(coops_cli(station,
                                date=date,
                                begin_date=begin_date,
                                end_date=end_date,
                                range=range,
                                product=product,
                                datum=datum,
                                time_zone=time_zone,
                                interval=interval,
                                bin=bin))


def coops(station,
          date=None,
          begin_date=None,
          end_date=None,
          range=None,
          product='water_level',
          datum='NAVD',
          time_zone='GMT',
          interval='h',
          bin=None):
    r"""Download Center for Operational Oceanographic Products and Services."""
    from tsgettoolbox.services import coops as placeholder
    r = resource(
        r'http://tidesandcurrents.noaa.gov/api/datagetter',
        station=station,
        date=date,
        begin_date=begin_date,
        end_date=end_date,
        range=range,
        product=product,
        datum=datum,
        units='metric',
        time_zone=time_zone,
        interval=interval,
        bin=bin,
    )
    return odo(r, pd.DataFrame)




import mando

try:
    from mando.rst_text_formatter import RSTHelpFormatter as HelpFormatter
except ImportError:
    from argparse import RawTextHelpFormatter as HelpFormatter

from tstoolbox import tsutils


@mando.command('cpc', formatter_class=HelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def cpc_cli(state=None,
            climate_division=None,
            start_date=None,
            end_date=None):
    r"""Access Climate Prediction Center, Weekly Drought Index dataset.

    Climate Prediction Center: http://www.cpc.ncep.noaa.gov/
    Weekly Drought Index: http://www.cpc.ncep.noaa.gov/products/analysis_monitoring/cdus/palmer_drought/

    Command Line ::

        tsgettoolbox cpc --state=FL --start_date 2017-01-01

    Python API ::

        df = tsgettoolbox.cpc("FL",
                              start_date="2017-01-01",
                              end_date="2017-02-01")

    Parameters
    ----------
    state : ``None`` or str
        [optional]

        If specified, results will be limited to the state corresponding to the
        given 2-character state code.

    climate_division : ``None`` or int
        [optional]

        If specified, results will be limited to the climate division.

    {start_date}

    {end_date}

    """
    tsutils._printiso(cpc(state=state,
                          climate_division=climate_division,
                          start_date=start_date,
                          end_date=end_date))


def cpc(state=None,
        climate_division=None,
        start_date=None,
        end_date=None):
    r"""Access Climate Prediction Center, Weekly Drought Index dataset."""
    from tsgettoolbox.services import cpc
    df = cpc.ulmo_df(state=state,
                     climate_division=climate_division,
                     start_date=tsutils.parsedate(start_date),
                     end_date=tsutils.parsedate(end_date))
    return df




from tsgettoolbox.odo import odo, resource
import pandas as pd
import mando

try:
    from mando.rst_text_formatter import RSTHelpFormatter as HelpFormatter
except ImportError:
    from argparse import RawTextHelpFormatter as HelpFormatter

from tstoolbox import tsutils


@mando.command('modis', formatter_class=HelpFormatter, doctype='numpy')
def modis_cli(lat,
              lon,
              product,
              band,
              startdate=None,
              enddate=None):
    r"""Download MODIS derived data.

    This data are derived data sets from MODIS satellite photos.

    MCD12Q1

    The MODIS Land Cover Type product contains five classification
    schemes, which describe land cover properties derived from
    observations spanning a year's input of Terra- and Aqua-MODIS
    data.  The primary land cover scheme identifies 17 land cover
    classes defined by the International Geosphere Biosphere
    Programme (IGBP), which includes 11 natural vegetation classes,
    3 developed and mosaicked land classes, and three non-vegetated
    land classes.

    The MODIS Terra + Aqua Land Cover Type Yearly L3 Global 500
    m SIN Grid product incorporates five different land cover
    classification schemes, derived through a supervised
    decision-tree classification method.

    V051 Land Cover Type products are produced with revised training
    data and certain algorithm refinements.  For further details,
    please consult the following paper:

    Friedl, M. A., Sulla-Menashe, D., Tan, B., Schneider, A.,
    Ramankutty, N., Sibley, A., and Huang, X. (2010). MODIS
    Collection 5 global land cover: Algorithm refinements and
    characterization of new datasets. Remote Sensing of Environment,
    114, 168-182.

    Land Cover Datasets

    +-------------------+----------+---------------------------+
    | Band              | Abbrev   | Description               |
    +===================+==========+===========================+
    | Land_Cover_Type_1 | IGBP     | global vegetation         |
    |                   |          | classification scheme     |
    +-------------------+----------+---------------------------+
    | Land_Cover_Type_2 | UMD      | University of Maryland    |
    |                   |          | scheme                    |
    +-------------------+----------+---------------------------+
    | Land_Cover_Type_3 | LAI/fPAR | MODIS-derived scheme      |
    +-------------------+----------+---------------------------+
    | Land_Cover_Type_4 | NPP      | MODIS-derived Net Primary |
    |                   |          | Production (NPP) scheme   |
    +-------------------+----------+---------------------------+
    | Land_Cover_Type_5 | PFT      | Plant Functional Type     |
    |                   |          | (PFT) scheme              |
    +-------------------+----------+---------------------------+

    Land Cover Types Description

    +------+-----------+-----------+-----------+-----------+-----------+
    | Code | IGBP      | UMD       | LAI/fPAR  | NPP       | PFT       |
    +======+===========+===========+===========+===========+===========+
    | 0    | Water     | Water     | Water     | Water     | Water     |
    +------+-----------+-----------+-----------+-----------+-----------+
    | 1    | Evergreen | Evergreen | Grasses/  | Evergreen | Evergreen |
    |      | Needle    | Needle    | Cereal    | Needle    | Needle    |
    |      | leaf      | leaf      | crop      | leaf      | leaf      |
    |      | forest    | forest    |           |           |           |
    +------+-----------+-----------+-----------+-----------+-----------+
    | 2    | Evergreen | Evergreen | Shrub     | Evergreen | Evergreen |
    |      | Broadleaf | Broadleaf |           | Broadleaf | Broadleaf |
    |      | forest    | forest    |           |           |           |
    +------+-----------+-----------+-----------+-----------+-----------+
    | 3    | Deciduous | Deciduous | Broadleaf | Deciduous | Deciduous |
    |      | Needle    | Needle    | crop      | Needle    | Needle    |
    |      | leaf      | leaf      |           | leaf      | leaf      |
    |      | forest    | forest    |           |           |           |
    +------+-----------+-----------+-----------+-----------+-----------+
    | 4    | Deciduous | Deciduous | Savanna   | Deciduous | Deciduous |
    |      | Broadleaf | Broadleaf |           | Broadleaf | Broadleaf |
    |      | forest    | forest    |           |           |           |
    +------+-----------+-----------+-----------+-----------+-----------+
    | 5    | Mixed     | Mixed     | Evergreen | Annual    | Shrub     |
    |      | forest    | forest    | Broadleaf | Broadleaf |           |
    |      |           |           | forest    |           |           |
    +------+-----------+-----------+-----------+-----------+-----------+
    | 6    | Closed    | Closed    | Deciduous | Annual    | Grassland |
    |      | shrubland | shrubland | Broadleaf | grass     |           |
    |      |           |           | forest    |           |           |
    +------+-----------+-----------+-----------+-----------+-----------+
    | 7    | Open      | Open      | Evergreen | Non-      | Cereal    |
    |      | shrubland | shrubland | Needle    | vegetated | crops     |
    |      |           |           | leaf      | land      |           |
    |      |           |           | forest    |           |           |
    +------+-----------+-----------+-----------+-----------+-----------+
    | 8    | Woody     | Woody     | Deciduous | Urban     | Broad-    |
    |      | savanna   | savanna   | Needle    |           | leaf      |
    |      |           |           | leaf      |           | crops     |
    |      |           |           | forest    |           |           |
    +------+-----------+-----------+-----------+-----------+-----------+
    | 9    | Savanna   | Savanna   | Non-      |           | Urban and |
    |      |           |           | vegetated |           | built-up  |
    +------+-----------+-----------+-----------+-----------+-----------+
    | 10   | Grassland | Grassland | Urban     |           | Snow and  |
    |      |           |           |           |           | ice       |
    +------+-----------+-----------+-----------+-----------+-----------+
    | 11   | Permanent |           |           |           | Barren or |
    |      | wetlands  |           |           |           | sparse    |
    +------+-----------+-----------+-----------+-----------+-----------+
    | 12   | Croplands | Cropland  |           |           |           |
    +------+-----------+-----------+-----------+-----------+-----------+
    | 13   | Urban and | Urban and |           |           |           |
    |      | built-up  | built-up  |           |           |           |
    +------+-----------+-----------+-----------+-----------+-----------+
    | 14   | Cropland  |           |           |           |           |
    |      | /Natural  |           |           |           |           |
    |      | mosaic    |           |           |           |           |
    +------+-----------+-----------+-----------+-----------+-----------+
    | 15   | Snow and  |           |           |           |           |
    |      | ice       |           |           |           |           |
    +------+-----------+-----------+-----------+-----------+-----------+
    | 16   | Barren or | Barren or |           |           |           |
    |      | sparsely  | sparsely  |           |           |           |
    |      | vegetated | vegetated |           |           |           |
    +------+-----------+-----------+-----------+-----------+-----------+
    | 254  | Unclassi  | Unclassi  | Unclassi  | Unclassi  | Unclassi  |
    |      | fied      | fied      | fied      | fied      | fies      |
    +------+-----------+-----------+-----------+-----------+-----------+
    | 255  | Fill      | Fill      | Fill      | Fill      | Fill      |
    |      | Value     | Value     | Value     | Value     | Value     |
    +------+-----------+-----------+-----------+-----------+-----------+



    Parameters
    ----------
    lat : float
        Latitude (required): Enter single geographic point by
        latitude.

    lon : float
        Longitude (required): Enter single geographic point by
        longitude.

    product : str
        One of the following values in the 'product'
        column.

        +----------+-------------------------------+--------+---------+
        | product  | Name                          | Freq.  | Size    |
        |          |                               | (Days) | (Meter) |
        +==========+===============================+========+=========+
        | MCD12Q1  | MODIS/Terra+Aqua Land Cover   | annual |  500    |
        |          | Type Yearly L3                |        |         |
        +----------+-------------------------------+--------+---------+
        | MCD12Q2  | MODIS/Terra+Aqua Land Cover   | annual |  500    |
        |          | Dynamics Yearly L3            |        |         |
        +----------+-------------------------------+--------+---------+
        | MCD15A2H | MODIS/Terra+Aqua Leaf Area    | 8      |  500    |
        |          | Index/FPAR 8-Day L4           |        |         |
        +----------+-------------------------------+--------+---------+
        | MCD15A3H | MODIS/Terra+Aqua Leaf Area    | 4      |  500    |
        |          | Index/FPAR 4-Day L4           |        |         |
        +----------+-------------------------------+--------+---------+
        | MOD09A1  | MODIS/Terra Surface           | 8      |  500    |
        |          | Reflectance 8-Day L3          |        |         |
        +----------+-------------------------------+--------+---------+
        | MOD11A2  | MODIS/Terra Land Surface      | 8      | 1000    |
        |          | Temperature/Emissivity        |        |         |
        |          | 8-Day L3                      |        |         |
        +----------+-------------------------------+--------+---------+
        | MOD13Q1  | MODIS/Terra Vegetation        | 16     |  250    |
        |          | Indices 16-Day L3             |        |         |
        +----------+-------------------------------+--------+---------+
        | MOD15A2H | MODIS/Terra Leaf Area         | 8      |  500    |
        |          | Index/FPAR 8-Day L4           |        |         |
        +----------+-------------------------------+--------+---------+
        | MOD17A2H | MODIS/Terra Gross Primary     | 8      |  500    |
        |          | Production 8-Day L4           |        |         |
        +----------+-------------------------------+--------+---------+
        | MOD17A3H | MODIS/Terra Net Primary       | annual |  500    |
        |          | Production Yearly L4          |        |         |
        +----------+-------------------------------+--------+---------+
        | MYD09A1  | MODIS/Aqua Surface            | 8      |  500    |
        |          | Reflectance 8-Day L3          |        |         |
        +----------+-------------------------------+--------+---------+
        | MYD11A2  | MODIS/Aqua Land Surface       | 8      | 1000    |
        |          | Temperature/Emissivity        |        |         |
        |          | 8-Day L3                      |        |         |
        +----------+-------------------------------+--------+---------+
        | MYD13Q1  | MODIS/Aqua Vegetation         | 16     |  250    |
        |          | Indices 16-Day L3             |        |         |
        +----------+-------------------------------+--------+---------+
        | MYD15A2H | MODIS/Aqua Leaf Area          | 8      |  500    |
        |          | Index/FPAR 8-Day L4           |        |         |
        +----------+-------------------------------+--------+---------+
        | MYD17A2H | MODIS/Aqua Gross Primary      | 8      |  500    |
        |          | Production 8-Day L4           |        |         |
        +----------+-------------------------------+--------+---------+
        | MYD17A3H | MODIS/Aqua Net Primary        | annual |  500    |
        |          | Production Yearly L4          |        |         |
        +----------+-------------------------------+--------+---------+

    band : str
        One of the following. The 'band' selected from the second column must
        match the 'product' in the first column.

        +----------+-----------------------------------------------+
        | product  | band                                          |
        +==========+===============================================+
        | MCD12Q1  | LC_Property_1 (not populated)                 |
        |          | LC_Property_2 (not populated)                 |
        |          | LC_Property_3 (not populated)                 |
        |          | Land_Cover_Type_1                             |
        |          | Land_Cover_Type_2                             |
        |          | Land_Cover_Type_3                             |
        |          | Land_Cover_Type_4                             |
        |          | Land_Cover_Type_5                             |
        |          | Land_Cover_Type_1_Assessment                  |
        |          | Land_Cover_Type_2_Assessment (not populated)  |
        |          | Land_Cover_Type_3_Assessment (not populated)  |
        |          | Land_Cover_Type_4_Assessment                  |
        |          | Land_Cover_Type_5_Assessment (not populated)  |
        |          | Land_Cover_Type_1_Secondary                   |
        |          | Land_Cover_Type_1_Secondary_Percent           |
        |          | (not populated)                               |
        +----------+-----------------------------------------------+
        | MCD12Q2  | NBAR_EVI_Onset_Greenness_Maximum.Num_Modes_02 |
        |          | NBAR_EVI_Onset_Greenness_Minimum.Num_Modes_02 |
        |          | NBAR_EVI_Onset_Greenness_Maximum.Num_Modes_01 |
        |          | NBAR_EVI_Onset_Greenness_Minimum.Num_Modes_01 |
        |          | Onset_Greenness_Minimum.Num_Modes_02          |
        |          | Onset_Greenness_Decrease.Num_Modes_02         |
        |          | Onset_Greenness_Maximum.Num_Modes_02          |
        |          | Onset_Greenness_Increase.Num_Modes_02         |
        |          | Onset_Greenness_Minimum.Num_Modes_01          |
        |          | Onset_Greenness_Decrease.Num_Modes_01         |
        |          | Onset_Greenness_Maximum.Num_Modes_01          |
        |          | Onset_Greenness_Increase.Num_Modes_01         |
        |          | NBAR_EVI_Area.Num_Modes_01                    |
        |          | NBAR_EVI_Area.Num_Modes_02                    |
        +----------+-----------------------------------------------+
        | MCD15A2H | FparExtra_QC                                  |
        |          | FparLai_QC                                    |
        |          | FparStdDev_500m                               |
        |          | LaiStdDev_500m                                |
        |          | Lai_500m                                      |
        |          | Fpar_500m                                     |
        +----------+-----------------------------------------------+
        | MCD15A3H | FparExtra_QC                                  |
        |          | FparLai_QC                                    |
        |          | FparStdDev_500m                               |
        |          | LaiStdDev_500m                                |
        |          | Lai_500m                                      |
        |          | Fpar_500m                                     |
        +----------+-----------------------------------------------+
        | MOD09A1  | sur_refl_day_of_year                          |
        |          | sur_refl_qc_500m                              |
        |          | sur_refl_raz                                  |
        |          | sur_refl_state_500m                           |
        |          | sur_refl_szen                                 |
        |          | sur_refl_vzen                                 |
        |          | sur_refl_b01                                  |
        |          | sur_refl_b02                                  |
        |          | sur_refl_b03                                  |
        |          | sur_refl_b04                                  |
        |          | sur_refl_b05                                  |
        |          | sur_refl_b06                                  |
        |          | sur_refl_b07                                  |
        +----------+-----------------------------------------------+
        | MOD11A2  | Clear_sky_days                                |
        |          | Clear_sky_nights                              |
        |          | Day_view_angl                                 |
        |          | Day_view_time                                 |
        |          | Emis_31                                       |
        |          | Emis_32                                       |
        |          | Night_view_angl                               |
        |          | Night_view_time                               |
        |          | QC_Day                                        |
        |          | QC_Night                                      |
        |          | LST_Day_1km                                   |
        |          | LST_Night_1km                                 |
        +----------+-----------------------------------------------+
        | MOD13Q1  | 250m_16_days_blue_reflectance                 |
        |          | 250m_16_days_MIR_reflectance                  |
        |          | 250m_16_days_NIR_reflectance                  |
        |          | 250m_16_days_pixel_reliability                |
        |          | 250m_16_days_red_reflectance                  |
        |          | 250m_16_days_relative_azimuth_angle           |
        |          | 250m_16_days_sun_zenith_angle                 |
        |          | 250m_16_days_view_zenith_angle                |
        |          | 250m_16_days_VI_Quality                       |
        |          | 250m_16_days_NDVI                             |
        |          | 250m_16_days_EVI                              |
        |          | 250m_16_days_composite_day_of_the_year        |
        +----------+-----------------------------------------------+
        | MOD15A2H | FparExtra_QC                                  |
        |          | FparLai_QC                                    |
        |          | FparStdDev_500m                               |
        |          | LaiStdDev_500m                                |
        |          | Lai_500m                                      |
        |          | Fpar_500m                                     |
        +----------+-----------------------------------------------+
        | MOD17A2H | Psn_QC_500m                                   |
        |          | PsnNet_500m                                   |
        |          | Gpp_500m                                      |
        +----------+-----------------------------------------------+
        | MOD17A3H | Npp_QC_500m                                   |
        |          | Npp_500m                                      |
        +----------+-----------------------------------------------+
        | MYD09A1  | sur_refl_day_of_year                          |
        |          | sur_refl_qc_500m                              |
        |          | sur_refl_raz                                  |
        |          | sur_refl_state_500m                           |
        |          | sur_refl_szen                                 |
        |          | sur_refl_vzen                                 |
        |          | sur_refl_b01                                  |
        |          | sur_refl_b02                                  |
        |          | sur_refl_b03                                  |
        |          | sur_refl_b04                                  |
        |          | sur_refl_b05                                  |
        |          | sur_refl_b06                                  |
        |          | sur_refl_b07                                  |
        +----------+-----------------------------------------------+
        | MYD11A2  | Clear_sky_days                                |
        |          | Clear_sky_nights                              |
        |          | Day_view_angl                                 |
        |          | Day_view_time                                 |
        |          | Emis_31                                       |
        |          | Emis_32                                       |
        |          | Night_view_angl                               |
        |          | Night_view_time                               |
        |          | QC_Day                                        |
        |          | QC_Night                                      |
        |          | LST_Day_1km                                   |
        |          | LST_Night_1km                                 |
        +----------+-----------------------------------------------+
        | MYD13Q1  | 250m_16_days_blue_reflectance                 |
        |          | 250m_16_days_MIR_reflectance                  |
        |          | 250m_16_days_NIR_reflectance                  |
        |          | 250m_16_days_pixel_reliability                |
        |          | 250m_16_days_red_reflectance                  |
        |          | 250m_16_days_relative_azimuth_angle           |
        |          | 250m_16_days_sun_zenith_angle                 |
        |          | 250m_16_days_view_zenith_angle                |
        |          | 250m_16_days_VI_Quality                       |
        |          | 250m_16_days_NDVI                             |
        |          | 250m_16_days_EVI                              |
        |          | 250m_16_days_composite_day_of_the_year        |
        +----------+-----------------------------------------------+
        | MYD15A2H | FparExtra_QC                                  |
        |          | FparLai_QC                                    |
        |          | FparStdDev_500m                               |
        |          | LaiStdDev_500m                                |
        |          | Lai_500m                                      |
        |          | Fpar_500m                                     |
        +----------+-----------------------------------------------+
        | MYD17A2H | Psn_QC_500m                                   |
        |          | PsnNet_500m                                   |
        |          | Gpp_500m                                      |
        +----------+-----------------------------------------------+
        | MYD17A3H | Npp_QC_500m                                   |
        |          | Npp_500m                                      |
        +----------+-----------------------------------------------+

    startdate
        ISO 8601 formatted date string

    enddate
        ISO 8601 formatted date string

    Notes
    -----
    Citation instructions are from https://modis.ornl.gov/citation.html

    When using subsets of MODIS Land Products from the ORNL DAAC, please
    use the following citation format:

    Citation: Single Site

    Format (single site):

    ORNL DAAC 2008. MODIS Collection 5 Land Products Global Subsetting
    and Visualization Tool. ORNL DAAC, Oak Ridge, Tennessee, USA.
    Accessed Month dd, yyyy. Subset obtained for [Product name] product
    at [Lat],[Lon], time period: [Start date] to [End date], and subset
    size: [Width] x [Height] km. http://dx.doi.org/10.3334/ORNLDAAC/1241

    Single site:

    ORNL DAAC. 2008. MODIS Collection 5 Land Products Global Subsetting
    and Visualization Tool. ORNL DAAC, Oak Ridge, Tennessee, USA.
    Accessed August 25, 2015. Subset obtained for MOD13Q1 product at
    39.497N,107.3028W, time period: 2000-02-18 to 2015-07-28, and subset
    size: 0.25 x 0.25 km. http://dx.doi.org/10.3334/ORNLDAAC/1241


    Citation: Multiple Sites

    Format (multiple sites, clustered together):

    ORNL DAAC 2008. MODIS Collection 5 Land Products Global Subsetting
    and Visualization Tool. ORNL DAAC, Oak Ridge, Tennessee, USA.
    Accessed Month dd, yyyy. Subset obtained for [Product name] product
    at various sites in Spatial Range: N=DD.DD, S=DD.DD, E=DDD.DD,
    W=DDD.DD, time period: [Start date] to [End date], and subset size:
    [Width] x [Height] km. http://dx.doi.org/10.3334/ORNLDAAC/1241

    Multiple sites, clustered together:

    ORNL DAAC. 2008. MODIS Collection 5 Land Products Global Subsetting
    and Visualization Tool. ORNL DAAC, Oak Ridge, Tennessee, USA.
    Accessed August 25, 2015. Subset obtained for MOD13Q1 product at
    various sites in Spatial Range: N=39.49N, S=39.25N, E=107.42W,
    W=106.48W, time period: 2000-02-18 to 2015-07-28, and subset size:
    0.25 x 0.25 km. http://dx.doi.org/10.3334/ORNLDAAC/1241
    "Spatial Range: N=DD.DD, S=DD.DD, E=DDD.DD, W=DDD.DD" is the
    bounding box for the site locations used for requesting subsets.

    Please cite each product separately.

    The coordinates used in the citation are the Latitude and Longitude
    (decimal degrees) specified by the user when the order is placed,
    trimmed to 4 decimal places.  The citation is also sent in the email
    along with data retrieval instructions after the order is processed.
    BibTeX (.bib) file is available for download on the data
    visualization and download page. Please modify it manually for
    multiple sites.
    """
    tsutils._printiso(modis(lat,
                            lon,
                            product,
                            band,
                            startdate=startdate,
                            enddate=enddate))


def modis(lat,
          lon,
          product,
          band,
          startdate=None,
          enddate=None):
    r"""Download MODIS derived data."""
    from tsgettoolbox.services import modis as placeholder

    r = resource(
        r'https://modis.ornl.gov/cgi-bin/MODIS/soapservice/MODIS_soapservice.wsdl',
        product=product,
        band=band,
        latitude=lat,
        longitude=lon,
        startdate=startdate,
        enddate=enddate)
    return odo(r, pd.DataFrame)



"""
This module provides direct access to `Climate Predicition Center`_.

`Weekly Drought Index`_ dataset from `Climate Prediction Center`_.

.. _Climate Prediction Center: http://www.cpc.ncep.noaa.gov/
.. _Weekly Drought Index: http://www.cpc.ncep.noaa.gov/products/analysis_monitoring/cdus/palmer_drought/
"""
from __future__ import print_function
from __future__ import absolute_import

import pandas as pd

from tsgettoolbox.ulmo.cpc.drought.core import get_data


def ulmo_df(state=None,
            climate_division=None,
            start_date=None,
            end_date=None):
    df = get_data(state=state,
                  climate_division=climate_division,
                  start=start_date,
                  end=end_date,
                  as_dataframe=True)
    df = df.set_index('period')
    df.index = pd.PeriodIndex(df.index)
    df.index.name = 'Datetime'
    return df


if __name__ == '__main__':
    r = ulmo_df(state='FL',
                climate_division=1,
                start_date='2017-01-01',
                end_date='2017-10-02')

    print('FL EVERYTHING')
    print(r)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from itertools import chain
import os
import os.path
import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

version = open('VERSION').readline().strip()

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist')
    os.system('twine upload dist/tsgettoolbox-{0}*'.format(version))
    sys.exit()

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()

install_requires = [
    # List your project dependencies here.
    # For more details, see:
    # http://packages.python.org/distribute/setuptools.html#declaring-dependencies
    'appdirs',
    'beautifulsoup4',
    'dask >= 0.11.1',
    'datashape >= 0.5.0',
    'future',
    'geojson',
    'html5lib<=0.9999999',
    'isodate',
    'lxml',
    'multipledispatch >= 0.4.7',
    'networkx >= 1.0',
    'networkx < 2.0',
    'numpy >= 1.7',
    'pandas >= 0.15.0',
    'pytest',
    'requests',
    'suds-jurko',
    'tables',
    'toolz >= 0.7.3',
    'tstoolbox >= 22.55.29.20',
    'zeep',
]


def read(filename):
    with open(filename, 'r') as f:
        return f.read()


def read_reqs(filename):
    return read(filename).strip().splitlines()


def extras_require():
    extras = {req: read_reqs('etc/requirements_%s.txt' % req)
              for req in {'aws',
                          'bcolz',
                          'bokeh',
                          'ci',
                          'h5py',
                          'mongo',
                          'mysql',
                          'postgres',
                          'pytables',
                          'sas',
                          'ssh',
                          'sql',
                          'test'}}

    extras['mysql'] += extras['sql']
    extras['postgres'] += extras['sql']

    # don't include the 'ci' or 'test' targets in 'all'
    extras['all'] = list(chain.from_iterable(v for k, v in extras.items()
                                             if k not in {'ci', 'test'}))
    return extras


setup(name='tsgettoolbox',
      version=version,
      description='Will get time series from different sources on the internet.',
      long_description=README,
      classifiers=['Environment :: Console',
                   'Intended Audience :: Developers',
                   'Intended Audience :: Science/Research',
                   'License :: OSI Approved :: BSD License',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python :: 2.7',
                   'Programming Language :: Python :: 3',
                   ],
      keywords='time_series uri url web_services rest',
      author='Tim Cera, P.E.',
      author_email='tim@cerazone.net',
      url='http://timcera.bitbucket.io/tsgettoolbox/docsrc/index.html',
      packages=['tsgettoolbox',
                'tsgettoolbox/services',
                'tsgettoolbox/services/lcra',
                'tsgettoolbox/services/ncdc',
                'tsgettoolbox/services/usace',
                'tsgettoolbox/services/usgs',
                'tsgettoolbox/functions',
                'tsgettoolbox/odo',
                'tsgettoolbox/ulmo',
                ],
      package_data={'tsgettoolbox': 'tsgettoolbox/services/usgs/*.dat'},
      include_package_data=True,
      zip_safe=False,
      install_requires=install_requires,
      extras_require=extras_require(),
      entry_points={
          'console_scripts':
              ['tsgettoolbox=tsgettoolbox.tsgettoolbox:main']
      },
      test_suite='tests',
      )




from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os
try:
    import ConfigParser as cp
except ImportError:
    import configparser as cp

from tsgettoolbox import appdirs

dirs = appdirs.AppDirs('tsgettoolbox', 'tsgettoolbox')


def read_api_key(service):
    # Read in API key
    if not os.path.exists(dirs.user_config_dir):
        os.makedirs(dirs.user_config_dir)
    configfile = os.path.join(dirs.user_config_dir, 'config.ini')
    if not os.path.exists(configfile):
        with open(configfile, 'w') as fp:
            fp.write("""

[{0}]
api_key = ReplaceThisStringWithYourKey

""".format(service))
    # Make sure read only by user.
    os.chmod(configfile, 0o600)

    inifile = cp.ConfigParser()
    inifile.readfp(open(configfile, 'r'))

    try:
        api_key = inifile.get(service, 'api_key')
    except:
        with open(configfile, 'a') as fp:
            fp.write("""

[{0}]
api_key = ReplaceThisStringWithYourKey

""".format(service))
        api_key = 'ReplaceThisStringWithYourKey'

    inifile.readfp(open(configfile, 'r'))
    api_key = inifile.get(service, 'api_key')
    if api_key == 'ReplaceThisStringWithYourKey':
        raise ValueError("""
*
*   Need to edit {0}
*   to add your API key that you got from {1}.
*
""".format(configfile, service))

    return api_key



"""
Provides data from the `Emergency Data Distribution Network`.

The `DCP message format`_ includes some header information that is parsed
and the message body, with a variable number of characters. The format of
the message body varies widely depending on the manufacturer of the
transmitter, data logger, sensors, and the technician who programmed the
DCP. The body can be simple ASCII, sometime with parameter codes and
time-stamps embedded, sometimes not. The body can also be in
'Pseudo-Binary' which is character encoding of binary data that uses 6 bits
of every byte and guarantees that all characters are printable.

.. _United States Geological Survey: http://www.usgs.gov/
.. _Emergency Data Distribution Network: http://eddn.usgs.gov/
.. _http://eddn.usgs.gov/dcpformat.html
"""
from tsgettoolbox.ulmo.usgs.eddn import decode
from tsgettoolbox.ulmo.usgs.eddn import get_data

# def get_data(
#         dcp_address, start=None, end=None, networklist='', channel='',
#         spacecraft='Any', baud='Any', electronic_mail='', dcp_bul='',
#         glob_bul='', timing='', retransmitted='Y', daps_status='N',
#         use_cache=False, cache_path=None, as_dataframe=True):


def ulmo_df(dcp_address,
            parser,
            start_date=None,
            end_date=None):
    first = get_data(dcp_address,
                     start=start_date,
                     end=end_date)
    return decode(first,
                  parser).sort_index()


if __name__ == '__main__':
    r = ulmo_df('C5149430',
                'twdb_sutron',
                start_date='P5D',
                end_date='P1D')

    print('FL EVERYTHING')
    print(r)



from __future__ import print_function
from __future__ import absolute_import

import logging
import os
from builtins import object
from collections import defaultdict
from io import BytesIO

from tsgettoolbox.odo import convert
from tsgettoolbox.odo import odo
from tsgettoolbox.odo import resource

import pandas as pd

import requests

from tstoolbox import tsutils

# NOAA/NOS Tides and Currents
# pd.read_csv('http://tidesandcurrents.noaa.gov/api/datagetter?
# begin_date=20130101&
# end_date=20130101&
# station=8454000&
# product=water_level&
# datum=mllw&
# units=metric&
# time_zone=gmt&
# application=web_services&
# format=csv',
#  index_col=0)


class NOS(object):
    def __init__(self,
                 url,
                 **query_params):
        params = {}
        params['interval'] = 'h'
        params['units'] = 'metric'
        params['time_zone'] = 'gmt'
        params['datum'] = 'mllw'
        params.update(query_params)

        params['begin_date'] = tsutils.parsedate(query_params['begin_date'],
                                                 strftime='%Y%m%d')
        params['end_date'] = tsutils.parsedate(query_params['end_date'],
                                               strftime='%Y%m%d')
        params['format'] = 'csv'
        params['application'] = 'tsgettoolbox'
        self.url = url
        self.query_params = params


# Function to make `resource` know about the new NOS type.
@resource.register(r'http(s)?://tidesandcurrents\.noaa\.gov.*', priority=17)
def resource_nos(uri, **kwargs):
    return NOS(uri, **kwargs)


# Function to convert from NOS type to pd.DataFrame
@convert.register(pd.DataFrame, NOS)
def nos_to_df(data, **kwargs):
    settings_map = defaultdict(lambda: [{'metric': '', 'english': ''}])
    settings_map['water_level'] = [
        {'metric': 'm', 'english': 'ft'}, 'h']    # Preliminary or verified
    # water levels, depending
    # on availability.
    settings_map['air_temperature'] = [
        {'metric': 'degC', 'english': 'degF'}, 'h']     # Air temperature
    # as measured at
    # the station.
    settings_map['water_temperature'] = [
        {'metric': 'degC', 'english': 'degF'}, 'h']  # Water temperature
    # as measured at
    # the station.
    settings_map['wind'] = [
        {'metric': 'm/s', 'english': 'ft/s'}, 'h']  # Wind speed,
    # direction, and gusts
    # as measured at the
    # station.
    settings_map['air_pressure'] = [
        {'metric': 'mb', 'english': 'mb'}, 'h']  # Barometric pressure as
    # measured at the station.
    settings_map['air_gap'] = [
        {'metric': 'm', 'english': 'ft'}, 'h']  # Air Gap (distance
    # between a bridge and the
    # water's surface) at the
    # station.
    settings_map['conductivity'] = [
        {'metric': 'mS/cm', 'english': 'mS/cm'}, 'h']  # The water's
    # conductivity as
    # measured at the
    # station.
    settings_map['visibility'] = [
        {'metric': 'km', 'english': 'nm'}, 'h']  # Visibility from the
    # station's visibility
    # sensor. A measure of
    # atmospheric clarity.
    settings_map['humidity'] = [
        {'metric': 'percent', 'english': 'percent'}, 'h']  # Relative
    # humidity as
    # measured at
    # the station.
    settings_map['salinity'] = [
        {'metric': 'PSU', 'english': 'PSU'}, 'h']  # Salinity and specific
    # gravity data for the
    # station.
    settings_map['hourly_height'] = [
        {'metric': 'm', 'english': 'ft'}, 'h']  # Verified hourly height
    # water level data for the
    # station.
    settings_map['high_low'] = [
        {'metric': 'm', 'english': 'ft'}, None]  # Verified high/low water
    # level data for the
    # station.
    settings_map['daily_mean'] = [
        {'metric': 'm', 'english': 'ft'}, None]  # Verified daily mean
    # water level data for
    # the station.
    settings_map['monthly_mean'] = [
        {'metric': 'm', 'english': 'ft'}, None]  # Verified monthly mean
    # water level data for
    # the station.
    settings_map['one_minute_water_level'] = [
        {'metric': 'm', 'english': 'ft'}, None]  # One minute water level
    # data for the station.
    settings_map['predictions'] = [
        {'metric': 'm', 'english': 'ft'}, 'h']  # 6 minute predictions
    # water level data for the
    # station.
    settings_map['datums'] = [
        {'metric': 'm', 'english': 'ft'}, None]  # datums data for the
    # currents stations.

    settings_map['currents'] = [
        {'metric': 'm/s', 'english': 'ft/s'}, 'h']  # Currents data for
    # currents stations.

    req = requests.get(data.url,
                       params=data.query_params)

    if os.path.exists('debug_tsgettoolbox'):
        logging.warning(req.url)
    req.raise_for_status()

    if b'Wrong ' in req.content:
        raise ValueError("""
*
*   The server responded with the error:
*   {0}
*
""".format(req.content))

    df = pd.read_csv(
        BytesIO(req.content),
        index_col=0,
        parse_dates=True)
    new_column_names = []
    for icolumn_name in df.columns:
        ncolumn_name = icolumn_name.lower().strip().replace(' ', '_')
        units = settings_map[ncolumn_name][0][data.query_params['units']]
        unitstr = ncolumn_name
        if units != '':
            unitstr = ':'.join([ncolumn_name, units])
        new_column_names.append(('NOS',
                                 data.query_params['station'],
                                 unitstr))
    df.columns = ['-'.join(i).rstrip('-') for i in new_column_names]
    time_zone_name = data.query_params['time_zone'].upper()
    if time_zone_name == 'GMT':
        time_zone_name = 'UTC'
    df = df.tz_localize(time_zone_name)
    df.index.name = 'Datetime:{0}'.format(time_zone_name)
    return df


if __name__ == '__main__':
    """
    http://tidesandcurrents.noaa.gov/api/datagetter?begin_date=20020101
    &end_date=20020102&range=1&station=8720218&product=water_level
    """

    r = resource(
        r'http://tidesandcurrents.noaa.gov/api/datagetter',
        begin_date='01/10/2002',
        end_date='2002-01-02',
        range=1,
        station='8720218',
        product='water_level')

    as_df = odo(r, pd.DataFrame)
    print('tidesandcurrents')
    print(as_df)



"""
Direct access to KBDI from `Texas Weather Connection`_.

Access the Texas `Daily Keetch-Byram Drought Index (KBDI)`_ dataset.

.. _Texas Weather Connection: http://twc.tamu.edu/
.. _Daily Keetch-Byram Drought Index (KBDI): http://twc.tamu.edu/drought/kbdi
"""
from __future__ import print_function

import pandas as pd

from tsgettoolbox.ulmo.twc.kbdi.core import get_data


def ulmo_df(county=None,
            start_date=None,
            end_date=None):

    return get_data(county=county,
                    start=pd.to_datetime(start_date),
                    end=pd.to_datetime(end_date),
                    as_dataframe=True)


if __name__ == '__main__':
    #    import time
    #
    #    r = ulmo_df('blah',
    #                'upperbasin')
    #
    #    print('UB EVERYTHING')
    #    print(r)
    #
    #    time.sleep(20)

    r = ulmo_df(48001,
                start_date='2015-11-04',
                end_date='2015-12-05')

    print('UB EVERYTHING')
    print(r)



from __future__ import print_function
from __future__ import absolute_import

import datetime
import logging
import os

from future import standard_library

standard_library.install_aliases()
from builtins import object
from builtins import range
from builtins import str

from tsgettoolbox.odo import convert
from tsgettoolbox.odo import odo
from tsgettoolbox.odo import resource

import pandas as pd

from tstoolbox import tsutils

try:
    import urllib.parse as urlp
except ImportError:
    import urllib as urlp

_units_map = {
    'tmax': ':degC',
    'tmin': ':degC',
    'srad': ':W/m2',
    'vp': ':Pa',
    'swe': ':kg/m2',
    'prcp': ':mm',
    'dayl': ':s',
}

# Daymet

# pd.read_csv('http://daymet.ornl.gov/data/send/saveData?measuredParams=tmax,tmin,prcp&lat=43.1&lon=-85.3&year=2000,2001',
# skiprows=7,
# date_parser=testp,
# index_col=0,
# parse_dates=[[0,1]])


class Daymet(object):
    def __init__(self, url, **query_params):
        avail_params = ['tmax', 'tmin', 'srad', 'vp', 'swe', 'prcp', 'dayl']
        params = {
            'measureParams': None,
            'year': None,
        }
        params.update(query_params)
        if params['measuredParams'] is None:
            params['measuredParams'] = ','.join(['tmax',
                                                 'tmin',
                                                 'srad',
                                                 'vp',
                                                 'swe',
                                                 'prcp',
                                                 'dayl',
                                                 ])
        else:
            for testparams in params['measuredParams'].split(','):
                if testparams not in avail_params:
                    raise ValueError("""
*
*   The measuredParams must be 'tmax', 'tmin', 'srad', 'vp', 'swe', 'prcp',
*   and 'dayl'.  You supplied {0}.
*
""".format(testparams))

        last_year = datetime.datetime.now().year - 1
        if params['year'] is None:
            params['year'] = ','.join([str(i)
                                       for i in range(1980, last_year + 1)])
        else:
            accumyear = []
            for testyear in params['year'].split(','):
                try:
                    iyear = int(tsutils.parsedate(testyear,
                                                  strftime='%Y'))
                    accumyear.append(iyear)
                except ValueError:
                    raise ValueError("""
*
*   The year= option must contain a comma separated list of integers.  You
*   supplied {0}.
*
""".format(testyear))
                if iyear < 1980 or iyear > last_year:
                    raise ValueError("""
*
*   The year= option must contain values from 1980 up to and including the last
*   calendar year.  You supplied {0}.
*
""".format(iyear))

            params['year'] = ','.join([str(i) for i in accumyear])

        self.url = url
        self.query_params = params

# Function to make `resource` know about the new Daymet type.


@resource.register(r'http(s)?://daymet\.ornl\.gov.*', priority=17)
def resource_daymet(uri, **kwargs):
    return Daymet(uri, **kwargs)


def _daymet_date_parser(year, doy):
    return pd.to_datetime('{}-{}'.format(int(float(year)),
                                         int(float(doy))),
                          format='%Y-%j')

# Function to convert from Daymet type to pd.DataFrame


@convert.register(pd.DataFrame, Daymet)
def daymet_to_df(data, **kwargs):
    req = urlp.unquote('{}?{}'.format(data.url,
                                      urlp.urlencode(data.query_params)))
    if os.path.exists('debug_tsgettoolbox'):
        logging.warning(req)
    df = pd.read_csv(req,
                     skiprows=7,
                     sep=',',
                     date_parser=_daymet_date_parser,
                     header=0,
                     index_col=0,
                     skipinitialspace=True,
                     parse_dates=[[0, 1]])
    tmpcols = [i.split()[0] for i in df.columns]
    df.columns = ['Daymet-{0}{1}'.format(i, _units_map[i]) for i in tmpcols]
    df.index.name = 'Datetime'
    return df


if __name__ == '__main__':
    r = resource(
        r'http://daymet.ornl.gov/data/send/saveData',
        measuredParams='tmax,tmin',
        lat=43.1,
        lon=-85.2,
        year='2000,2001'
    )

    as_df = odo(r, pd.DataFrame)
    print('Daymet')
    print(as_df)

    r = resource(
        r'http://daymet.ornl.gov/data/send/saveData',
        measuredParams=None,
        lat=43.1,
        lon=-85.2,
        year='3 years ago,2 years ago'
    )

    as_df = odo(r, pd.DataFrame)
    print('Daymet')
    print(as_df)
